
# HardyL1.py
# Andrew Davison, ad@coe.psu.ac.th, Nov. 2024
# David Wells - "Prime Numbers": Hardy-Littlewood conjectures
# https://en.wikipedia.org/wiki/First_Hardy%E2%80%93Littlewood_conjecture
# https://mathworld.wolfram.com/k-TupleConjecture.html
# Also see twinsPlot.py
'''
The First Hardy-Littlewood Conjecture is an 
asymptotic formula for counting prime pairs (p,p+2), or 
more generally, counting prime gaps of size k. 

The twin prime conjecture is:
π2(x)  == 2*C* integral 2 to x of dt/(log ⁡t)^2
where:
 * π2(x) is the number of twin primes (primes p and p+2) 
   up to x,
 * C is the twin primes constant

Also see twinsPlot.py
'''

import math
import pUtils

C = 0.6601618158  # Twin primes constant
  # https://mathworld.wolfram.com/TwinPrimesConstant.html


def countTwins(limit):
  ps = pUtils.primes(limit)
  count = 0
  for i in range(1, len(ps)-1):
    if ps[i] - ps[i-1] == 2:
      count += 1
  return count


def twin_prime_approx(limit):
  return 2 * C * sum( 1/(math.log(t)**2) 
                  for t in range(2, limit+1))


limit = 1000000
count = countTwins(limit)
approx = twin_prime_approx(limit)
print(f"Number of twin primes up to {limit}: {count}")
print(f"Hardy-Littlewood approx: {approx:.2f}")
