
# HardyL2.py
# Andrew Davison, ad@coe.psu.ac.th, Nov. 2024
# David Wells - "Prime Numbers": Hardy-Littlewood conjectures
# https://en.wikipedia.org/wiki/Second_Hardy%E2%80%93Littlewood_conjecture
# https://mathworld.wolfram.com/Hardy-LittlewoodConjectures.html

'''
The Second Hardy-Littlewood Conjecture suggests that:
   π(x+y) ≤ π(x) + π(y)
for all x, y ≥ 2 where π(x) is the prime-counting function, 
representing the number of primes less than or equal to x.
'''

import pUtils
import matplotlib.pyplot as plt

limit = 200
diffs = []
for x in range(limit+1):
  row = []
  for y in range(limit+1):
    diff = (pUtils.pi(x) + pUtils.pi(y)) - pUtils.pi(x+y)
    row.append(diff) 
  diffs.append(row)


hm=plt.imshow(diffs, cmap='rainbow', origin='lower')
plt.colorbar(hm)  
plt.xlabel('x')
plt.ylabel('y')

plt.title("$(\pi(x) + \pi(y)) - \pi(x+y)$")
plt.show()