
# abc.py
# Andrew Davison, ad@coe.psu.ac.th, Nov. 2024
# inputs: 5 27   or  4 127   or  3 125
# David Wells - "Prime Numbers": abc conjecture
# https://en.wikipedia.org/wiki/Abc_conjecture
'''
The ABC conjecture states that for any three positive 
integers a, b, and c such that a+b=c, and the integers 
a, b, and c are coprime, the product of the distinct 
prime factors of a, b, and c, denoted as rad(abc), is 
generally not much smaller than c.

If we calculate r(abc)^n/c, where n is any number 
greater than 1, then we can’t make r(abc)/c as small as 
we like, and this is true even if n is only slightly 
greater than 1. 

So r(abc)^n/c has a lower limit that isn’t zero. 
'''

import math
import pUtils


def rad(n):
  return math.prod( pUtils.primeFactors(n))


def isABC(a, b, c, epsilon=0.1):
  if math.gcd(a, b) != 1:
    return False
  abcRad = rad(a*b*c)
  p = abcRad ** (1 + epsilon)
  print(f"c = {c}, rad(abc)^(1+epsilon) = {p:.4f}")
  return p/c >= 1


a, b = map(int, input("coprimes a b? ").split())
res= isABC(a, b, a+b)
print(f"ABC Conjecture holds?", res)
