
# benford.py
# Andrew Davison, ad@coe.psu.ac.th, Nov. 2024
# David Wells - "Prime Numbers": Benford's law
# https://en.wikipedia.org/wiki/Benford%27s_law
# https://t5k.org/notes/faq/BenfordsLaw.html

import math
import pUtils
import matplotlib.pyplot as plt
from collections import Counter


limit = 100000
prims = pUtils.primes(limit)
leadingDigits = [int(str(p)[0]) for p in prims]
digitCounts = Counter(leadingDigits)

# Normalize
digs = range(1,10)
totPs = sum(digitCounts.values())
digitFreq = [digitCounts[d] / totPs for d in digs]

# Benford's Law expected distribution
benfordDist = [math.log10(1 + 1/d) for d in digs]


fig, ax = plt.subplots()

ax.bar(digs, digitFreq, label='Primes')
ax.plot(digs, benfordDist, 'r-', marker='o', 
           label="Benford's Law", lw=2)

ax.set_xticks(digs)
ax.set_xlabel('Leading Digit')
ax.set_ylabel('Proportion')
ax.set_title("Leading Digit Dist: Primes vs Benford's Law")
ax.legend()
plt.show()

