
# bernoulli.py
# Andrew Davison, ad@coe.psu.ac.th, Nov. 2024
# David Wells - "Prime Numbers": Benford's law
# https://en.wikipedia.org/wiki/Bernoulli_number

from fractions import Fraction as F
import math


def bernoulli_number(n):
  # compute a single Bernoulli number, B_n
  A = [0] * (n+1)
  for m in range(n+1):
    A[m] = F(1, m+1)
    for j in range(m, 0, -1):
      A[j-1] = j*(A[j-1] - A[j])
  return A[0]  # B_n


def bernoullis(n):
  ''' All the Bernoulli numbers up to n
    computed using a recursive relation:
    B_m = −∑ k=0 to m−1 (m k) B_k/(m−k+1)
  '''
  bs = [F(1,1)]  # B_0 = 1
  for m in range(1, n):
    tot = F(0,1)
    for k in range(m):
      tot += math.comb(m,k) * bs[k]/(m-k+1)
    bs.append(-tot)
  return bs



if __name__ == "__main__":
  limit = 19
  print("\nFirst", limit, "Bernoulli numbers:")
  bs = bernoullis(limit)
  for i in range(limit):
    print(f"B_{i:02d} = {str(bernoulli_number(i)):>10} {str(bs[i]):>10}")

