
# bias.py
# https://en.wikipedia.org/wiki/Chebyshev's_bias
# Also see littlewoodRace.py
'''
  Most of the time, there are more primes of the 
  form 4k + 3 than of the form 4k + 1, up to the 
  same limit.
'''

import math
import pUtils
import matplotlib.pyplot as plt

def pi(x, n, m):
  # Count number of primes of the form n*k+m up to x
  count = 0
  k = 0
  while True:
    num = n*k + m
    if num > x:
      break
    if pUtils.isPrime(num):
      count += 1
    k += 1
  return count

'''
x = 100
n = 5
m = 2
print(f"No. of primes of the form {n}*k+{m} up to {x}: {pi(x,n,m)}")
'''

limit = 10000
print(f"Range of x: 1-{limit}:")
xs = range(1, limit+1)
ps43 = []
ps41 = []
for x in xs:
  ps43.append(pi(x, 4, 3))
  ps41.append(pi(x, 4, 1))

print("Last pi(x,4,1), pi(x,4,3), 1/2*x*log x:")
print(f"        {ps43[-1]}     {ps41[-1]}        {0.5*limit/math.log(limit):.2f}")

diffs = [ps43[i] - ps41[i] for i in range(len(ps43))]

# plt.figure(figsize=(10, 4))
plt.plot(xs, diffs)
plt.title('Diff of Prime Counts: pi(x,4,3) - pi(x,4,1)')
plt.xlabel('x')
plt.ylabel('Difference')
plt.grid()
plt.show()
