
# brun.py
# Andrew Davison, ad@coe.psu.ac.th, Nov. 2024
# David Wells - "Prime Numbers": Brun's constant
# https://en.wikipedia.org/wiki/Brun%27s_theorem
# https://mathworld.wolfram.com/BrunsConstant.html

'''
The number obtained by adding the reciprocals of the 
odd twin primes,
B = (1/3+1/5) + (1/5+1/7) + (1/(11)+1/(13)) + 
    (1/(17)+1/(19)) +.... 	

By Brun's theorem, the series converges to a definite 
number, which expresses the scarcity of twin primes, 
even if there are infinitely many of them
'''

import pUtils
import matplotlib.pyplot as plt

BRUN = 1.902160578
limit = 10**8
STEP_SZ = 1000

print("Generating primes up to", limit, "...")
prims = pUtils.primes(limit)
tot2P = 0; numPairs = 0
tots2P = []
step = 0
for i in range(len(prims)-1):
  if (prims[i+1] - prims[i]) == 2: # twins
    # print(prims[i], prims[i+1])
    tot2P += (1/prims[i] + 1/prims[i+1])
    numPairs += 1
    if i%STEP_SZ == 0:
      tots2P.append(tot2P)
tots2P.append(tot2P)
print(f"Sum for {numPairs} pairs: {tot2P:.6f}")  
                    # 1.7383570439172515

# print(tots2P)
plt.plot(range(len(tots2P)), tots2P)
plt.axhline(y=BRUN, color='red')
plt.text(100, BRUN-0.01, "Brun's Constant")
plt.xlabel(f"no. of pairs*{STEP_SZ}")
plt.ylabel("total")
plt.title("The convergence to Brun's constant")
plt.show()
