
# champernowne.py
# Andrew Davison, ad@coe.psu.ac.th, Nov. 2024
# David Wells - "Prime Numbers": Champernowne's constant
# https://en.wikipedia.org/wiki/Champernowne_constant
# Also see concatPrimes.py
# input: 54


from decimal import Decimal as D
from decimal import getcontext


def dec2Base(n, base, precision=50):
  getcontext().prec = precision
  intPart = int(n)
  fracPart = n - D(intPart)
  intBase = int2Base(intPart, base)
  fracBase = frac2Base(fracPart, base, precision)
  return intBase + ('.' + fracBase if fracBase else '')


def int2Base(num, base):
  # Convert integer part
  if num == 0:
    return '0'
  ds = []
  while num:
    ds.append(int(num % base))
    num //= base
  # Convert ds > 9 to A, B, C, etc.
  return ''.join(reversed([str(d) if d < 10 
                 else chr(ord('A') + d-10) for d in ds]))


def frac2Base(frac, base, precision):
  # Convert fractional part
  ds = []
  while frac and len(ds) < precision:
    frac *= base
    d = int(frac)
    ds.append(d)
    frac -= d
  return ''.join([str(d) if d < 10 
                  else chr(ord('A') + d-10) for d in ds])


def champernowne(n):  # base 10
  s = ''.join(str(i) for i in range(1, n*2))  # gen extras
  return D("0." + s[:n])  # cut to n decimal places



n = int(input("no. decimal places n? "))
champ = champernowne(n)
print(champ)
base = 9
result = dec2Base(champ, base)
print(f"In base {base}:", result)




