
# consecShare.py
# Andrew Davison, ad@coe.psu.ac.th, Nov. 2024
'''
Prime numbers larger than 5 must end in 1, 3, 7 or 9 
(else they would be divisible by either 2 or 5), and 
it seems that a prime ending in any of these is less 
likely to be followed by another prime ending in the 
same digit.

https://scipython.com/blog/do-consecutive-primes-avoid-sharing-the-same-last-digit/
'''

import pUtils
import math
import matplotlib.pyplot as plt
from mpl_toolkits.mplot3d import Axes3D


lastDigs = [1,3,7,9]
nElems = len(lastDigs)*len(lastDigs)

digitCounts = {1: {1: 0, 3: 0, 7: 0, 9: 0},
               3: {1: 0, 3: 0, 7: 0, 9: 0},
               7: {1: 0, 3: 0, 7: 0, 9: 0},
               9: {1: 0, 3: 0, 7: 0, 9: 0}}

primes = pUtils.primes(11, 1000000)
for i in range(len(primes)-1):
  digitCounts[primes[i]%10][primes[i+1]%10] += 1

for key,counts in digitCounts.items():
  print(key, ":", end = '{ ')
  for k, v in counts.items():
    print(f" {k:1d}: {v:4d}, ", end = '')
  print("}")

xs = []
ys = []
zs = [0]*nElems  # Start of the bars at z=0
dzs = []

for d1 in lastDigs:
  tot = sum(digitCounts[d1].values())
  for d2 in lastDigs:
    xs.append(d1); ys.append(d2)
    height = int(round(digitCounts[d1][d2] / tot*100))
    dzs.append(height)

tickPos = [i + 0.5 for i in lastDigs]
mx = max(dzs)
colors = plt.cm.jet([dz/mx for dz in dzs])

fig = plt.figure()
ax = fig.add_subplot(111, projection='3d')
ax.bar3d(xs, ys, zs, 1, 1, dzs, color=colors)
ax.set_xlabel('Digit of First Prime')
ax.set_ylabel('Digit of Second Prime')
plt.xticks(tickPos, lastDigs)
plt.yticks(tickPos, lastDigs)
ax.set_zlabel('Digit Pair Count %')

plt.title("Final digits of consecutive primes")
plt.show()

