
# coprimePi.py
# Andrew Davison, ad@coe.psu.ac.th, Nov. 2024
# https://en.wikipedia.org/wiki/Coprime_integers
#   - see the "Probability of coprimality" section
# https://www.findingpi.org/co-primes.html
# Also see eulerBasel.py
import math, random

MAX_NUM = 10000000000

def estimatePi(nPairs):
  count = 0
  for i in range(0, nPairs):
    n1 = random.randint(1, MAX_NUM)
    n2 = random.randint(1, MAX_NUM)
    # increment the counter if both integers are coprimes
    if math.gcd(n1, n2) == 1:
      count += 1
  prob = float(count)/nPairs   
  return math.sqrt(6 / prob)

nPairs = 1000000
print("Generating", nPairs, "random pairs")
print(" pi est:", estimatePi(nPairs))
print("math.pi:", math.pi)
