
# cunnChain.py
# Andrew Davison, ad@coe.psu.ac.th, Nov. 2024
# inputs:  2  or  89  or  19
# David Wells - "Prime Numbers": Cunningham chains
# https://en.wikipedia.org/wiki/Cunningham_chain
'''
There are two types of Cunningham chains: 

* Cunningham chain of the first kind: Each prime 
p_(i+1) is obtained by the formula p_(i+1) = 2*p_i + 1 

* Cunningham chain of the second kind: Each prime 
p_(i+1) is obtained by the formula p_(i+1) =2*p_i - 1
'''

import operator
import pUtils

def cunningham1(p):
  return cunnChain(p, operator.add)

def cunningham2(p):
  return cunnChain(p, operator.sub)


def cunnChain(p, op):
  chain = [p]
  while True:
    num = op(2 * chain[-1], 1)  # + or -
    if pUtils.isPrime(num):
      chain.append(num)
    else:
      break
  return chain


start = int(input("starting prime? "))
chain1 = cunningham1(start)
print("Cunningham chain of the first kind:", chain1)
chain2 = cunningham2(start)
print("Cunningham chain of the second kind:", chain2)

