
# decRecur.py
# Andrew Davison, ad@coe.psu.ac.th, Nov. 2024
# David Wells - "Prime Numbers": decimals, recurring (periodic)
# https://en.wikipedia.org/wiki/Full_reptend_prime
# Also see repunit.py
# input:  13  or  17  or  29

import pUtils


def recurPeriod(numer, denom):
  remdPos = {}  # to store remainders and their pos
  remd = numer % denom
  pos = 0
  digits = []
  # do long division until a repeating remainder 
  # is found or remainder becomes 0
  while remd != 0:
    # at the start of the recurring period
    if remd in remdPos:
      start = remdPos[remd]
      rdigits = ''.join(digits[start:])
      return pos - start, rdigits
    remdPos[remd] = pos
    # do the long division step: get the next digit
    remd *= 10
    digit = remd // denom
    digits.append(str(digit))
    remd = remd % denom
    pos += 1
  return 0, ""   # no repeating period



denom = int(input("denominator? "))
numer = 1
print("Decimal:", 1/denom)
period, digits = recurPeriod(numer, denom)
print(f"The period for {numer}/{denom} is {period}")
if period > 0:
    print(f"The recurring digits are: {digits}")
else:
    print("There are no recurring digits.")

# -------------------------
limit = 100
prims = pUtils.primesNum(limit)

print(f"\nFull reptend primes in first {limit}:")
count = 0
for p in prims:
  period, _ = recurPeriod(numer, p)
  if period == p-1:
    print(p, end=' ')
    count += 1
print()
print("Count:", count)




