
# dickson.py
# Andrew Davison, ad@coe.psu.ac.th, Nov. 2024
# David Wells - "Prime Numbers": Dickson's conjecture
# https://en.wikipedia.org/wiki/Dickson%27s_conjecture
# https://math.stackexchange.com/questions/2930958/concerning-dicksons-conjecture
# input:  3  or  5

import math, random
import pUtils

NUMTESTS = 30
MAXCOEFF = 50   # Max coef for 'a' in f(n) = a*n + b
MAXCONST = 100   # Max constant for 'b'


def divsSmallPrimes(val, ps):
  for p in ps:
    if val%p == 0:
      # print(f"Div error for {p}")
      return True
  return False


def genPolys(nPolys):
  # Generate random polys of form f(n) = a*n + b
  ps = pUtils.primes(nPolys+1)
  polys = []
  numDivErrs = 0
  for _ in range(nPolys):
    while True:
      a = random.randint(1, MAXCOEFF)
      b = random.randint(1, MAXCONST)
      divP = divsSmallPrimes(a+b, ps)
      if not divP:
        if math.gcd(a, b) == 1:
          break  # success
      else:
        numDivErrs += 1
    polys.append((a, b))
  print("No. of div errors for small primes:", numDivErrs)
  return polys
 

nPolys = int(input("No. of polys? "))
polys = genPolys(nPolys)
print("Polys:", polys)

print(f"Testing the polys {NUMTESTS} times with increasing n values...")
n = 0
for t in range(NUMTESTS):
  allPrime = False
  while not allPrime:
    n += 1
    allPrime = True
    for a, b in polys:
      val = a*n + b
      if not pUtils.isPrime(val):
        allPrime = False
        break
  print(f"Test {t+1}. Succeeded for n = {n}")
