
# digitRepeats.py
# Andrew Davison, ad@coe.psu.ac.th, Nov. 2024
# https://en.wikipedia.org/wiki/Digital_root
'''
  Video: "Digital Roots and Prime Numbers", 
  G. Chang
  https://www.youtube.com/watch?v=SY_qBCV6b_A
'''

import pUtils
from collections import defaultdict


def digital_root(n):
  # digital root of n (base 10)
  return 1 + (n-1)%9


def findRepeats(s, n):
  # store substrings and their counts
  ssCount = defaultdict(int)

  # count all n-length substrings
  for i in range(len(s) - n + 1):
    ssCount[s[i:i+n]] += 1

  # get repeated substrings
  reps = {ss: count for ss, count in ssCount.items() 
                         if count > 1}
  
  # Sort by descending number of repeats
  return sorted(reps.items(), key=lambda x: x[1], 
                                   reverse=True)


def getMaxRepeats(reps):
  # collect all the strings that appear the most often
  if reps == []:
    return [], 0
  maxCount = reps[0][1]
  subs = []
  for ss, count in reps:
    if count < maxCount:
      break
    subs.append(ss)
  return subs, maxCount



limit = 1000000

print(f"Prime in range 5 to {limit}")  # to skip 3
primes = pUtils.primes(5, limit)

print("First primes:", *primes[:15], "...")
primeDRs = [digital_root(p) for p in primes]

print("First digital roots:", *primeDRs[:15], "...")
drStr = ''.join(map(str,primeDRs))

print("Start of digital roots string:", drStr[:25], "...")

print("\n(Substring length, max count, no. of strings with max count):")
for i in range(4,20):
  reps = findRepeats(drStr, i)
  substrs, maxCount = getMaxRepeats(reps)
  print(f"({i}, {maxCount}, {len(substrs)})", end=' ')
print()

print("\nSubstrings of length 5")
substrs, maxCount = getMaxRepeats(findRepeats(drStr, 5))
if len(substrs) == 1:
  print(f"  {substrs[0]} appears {maxCount} times")
else:
  print(f"  Each appears {maxCount} times:", *substrs)
