
# dirichlet.py
# Andrew Davison, ad@coe.psu.ac.th, Nov. 2024
# David Wells - "Prime Numbers": Dirichlet's theorem and primes in arithmetic series
# https://en.wikipedia.org/wiki/Dirichlet%27s_theorem_on_arithmetic_progressions
# inputs: 3 4   or  1 10   or  1 4

'''
Dirichlet prime number theorem, states that for any two 
positive *coprime* integers a and b, there are infinitely 
many primes of the form a + nb, where n is also a positive 
integer. In other words, there are infinitely many primes 
that are congruent to a modulo b. 

Fnding primes in an arithmetic progression involves identifying prime numbers that fit the form of an 
arithmetic sequence:
a, a+b, a+2b,... , a+(n−1)b
where:
  a is the first term,
  b is the common difference,
  n is the number of terms in the progression.
'''

import math, sys
import pUtils

def primesProgression(a, b, limit):
  prims = []
  i = 0
  while True:
    val = a + i*b
    if val > limit:
      break
    if pUtils.isPrime(val):
      prims.append(val)
    i += 1
  return prims


a, b = map(int, input("start step? ").split())

if math.gcd(a, b) != 1:
  print(f"{a} and {b} must be coprime")
  sys.exit()

limit = 300  # Limit for prime search
print(f"Primes in series up to {limit}:")
prims = primesProgression(a, b, limit)
print(*prims)

# ------------------------
a = 3
b = 4
print("\na, b:", a, b)
print(" Limit nPrimes approx")
for limit in [100, 1000, 10000, 100000, 1000000]:
  ps = primesProgression(a, b, limit)
  numApprox = round(limit / (pUtils.phi(a)* math.log(limit)), 1)
  print(f"{limit:7d} {len(ps):5d} {numApprox:7.1f}")

