
# divisorsSumPlot.py
# Andrew Davison, ad@coe.psu.ac.th, Nov. 2024
# David Wells - "Prime Numbers": divisors (factors)
# https://en.wikipedia.org/wiki/Divisor_function
# https://mathworld.wolfram.com/DivisorFunction.html
'''
Plots  n on the x-axis and the sum of proper divisors 
sigma(n) - n  on the y-axis. 

sigma(n) is the sum of all the divisors of n, 
optionally with each value taken to a power

Points tend to cluster around lines of rational slope 
-- some lines are indicated above with their slopes. 

Points on the 1.0 line correspond to perfect numbers. Above 
are the abundant numbers and below are the deficient numbers.
'''

import pUtils
import matplotlib.pyplot as plt


def drawLine(plt, ns, gradient):
  plt.plot(ns, [n*gradient for n in ns], lw=2, 
                alpha=0.5, color='red')
  x = ns[-1]
  plt.text(x, x*gradient, f' {gradient:.2f}', 
          color='blue', fontsize=8, ha='left')


ns = list(range(1, 1001))
sigs = [pUtils.sigma(n)-n for n in ns]

plt.figure(figsize=(10, 4))
# lines
# [1/10, 1/5, 1/3, 1/2, 2/3, 3/4, 1, 4/3, 3/2, 2]:
for grad in [1/5, 1/3, 1/2, 3/4, 1]:
  drawLine(plt, ns, grad)

plt.scatter(ns, sigs, s=4)

plt.title('$\sigma(n)-n$ vs. n with lines')
plt.xlabel('n')
plt.ylim(0, 2000)
plt.grid(True)
plt.show()
