
# economical.py
# Andrew Davison, ad@coe.psu.ac.th, Nov. 2024
# David Wells - "Prime Numbers": economical numbers
# https://en.wikipedia.org/wiki/Frugal_number
# https://en.wikipedia.org/wiki/Extravagant_number

import pUtils


def isEconomical(n):
  return (factsDigits(n) <= countDigits(n))

def isFrugal(n):
  return (factsDigits(n) < countDigits(n))

def isExtravagant(n):
  return (factsDigits(n) > countDigits(n))


def factsDigits(n):
  pfs = pUtils.factorizeDict(n)
  count = 0
  for prime, exp in pfs.items():
    count += countDigits(prime)
    if exp > 1:
      count += countDigits(exp)
  return count

def countDigits(n):
  return len(str(n))


print("Economicals in 2-50:")
for i in range(2, 51):
  if isEconomical(i):
    print(i, end=' ')
print()

print("\nEconomicals in 157-170:")
for i in range(157, 171):
  if isEconomical(i):
    print(i, end=' ')
print()

print("\nFrugals in 120-250:")
for i in range(120, 251):
  if isFrugal(i):
    print(i, end=' ')
print()

print("\nExtravagants in 2-20:")
for i in range(2, 21):
  if isExtravagant(i):
    print(i, end=' ')
print()

