
# eratoVisual.py
# Andrew Davison, ad@coe.psu.ac.th, Nov. 2024
# David Wells - "Prime Numbers": Eratosthenes of Cyrene, the sieve of
# https://en.wikipedia.org/wiki/Sieve_of_Eratosthenes
'''
  For a non-aminated version of the sieve, see
  primes() in pyprimes.py

  Uses pygame (https://www.pygame.org/docs/)

'''

import os
os.environ['PYGAME_HIDE_SUPPORT_PROMPT'] = "hide"
import pygame
 
# some colors
BLACK = (   0,   0,   0)
WHITE = ( 255, 255, 255)
GRAY =  ( 120, 120, 120)
LIGHT_BLUE = (138, 201, 235)

NUM_COLS = 20
NUM_ROWS = 20
CELL_SIZE = 28


def updateGrid(p):
  y = p // NUM_ROWS
  x = p % NUM_COLS
  if grid[y][x] > 0:    # p not excluded
    print(f"\nRemoving multiples of {p}:", end=' ')
    for i in range(p*p, NUM_ROWS*NUM_COLS, p):
       print(i, end = ' ')
       y = i // NUM_ROWS
       x = i % NUM_COLS
       if grid[y][x] > 0:  # not excluded yet
         grid[y][x]  = -1*grid[y][x]  # exclude
    print()


def drawGrid(screen):
  screen.fill(WHITE)
  size = NUM_COLS*CELL_SIZE
  
  for y in range(NUM_ROWS):
    for x in range(NUM_COLS):
      drawCell(screen, x, y, grid[y][x])

  # draw borders
  for x in range(NUM_COLS):
    pygame.draw.line(screen, BLACK, 
         (x*CELL_SIZE, 0), (x*CELL_SIZE, size), 2)
  for y in range(NUM_ROWS):
    pygame.draw.line(screen, BLACK, 
         (0, y*CELL_SIZE), (size, y*CELL_SIZE), 2)



def drawCell(screen, x, y, val):
  rect = pygame.Rect(x*CELL_SIZE, y*CELL_SIZE, 
                     CELL_SIZE, CELL_SIZE)

  if val <= 0:   # means value is excluded
    pygame.draw.rect(screen, WHITE, rect)
    msgPic = msgFont.render(str(-1*val), True, GRAY)
  else:
    pygame.draw.rect(screen, LIGHT_BLUE, rect)
    msgPic = msgFont.render(str(val), True, BLACK)

  msgRect = msgPic.get_rect()
  msgRect.center = rect.center
  screen.blit(msgPic, msgRect)
   


# ---------- main -------------

# initialize grid[y][x]
grid = [[(y*NUM_ROWS+x) for x in range(NUM_COLS)] 
                             for y in range(NUM_ROWS)]
grid[0][0] =  0   # 0 and 1 are not primes;
grid[0][1] = -1   # value <= 0 means not drawn


pygame.init()
screen = pygame.display.set_mode([NUM_COLS*CELL_SIZE,
                                  NUM_ROWS*CELL_SIZE])
screen.fill(WHITE)
pygame.display.set_caption("Sieve of Eratosthenes")

msgFont = pygame.font.SysFont("Arial", 16)
sieveFinished = False
p = 2
running = True  
while running:
  # handle events
  for event in pygame.event.get():
    if event.type == pygame.QUIT: 
      running = False
  
  # update
  if not sieveFinished:
    if p*p < NUM_COLS*NUM_ROWS:
      updateGrid(p)
      p += 1
    else:
      print("Sieveing finished")
      sieveFinished = True

  # redraw
  screen.fill(WHITE)  
  drawGrid(screen)
  pygame.display.update()
  if not sieveFinished:
    pygame.time.delay(1000)

pygame.quit()

