
# erdosConv.py
# Andrew Davison, ad@coe.psu.ac.th, Nov. 2024
# David Wells - "Prime Numbers": Erdos, Paul
# https://mathoverflow.net/questions/313999/a-problem-of-erd%C5%91s-on-convergence-of-sum-1nn-p-n-and-equidistribution-of
# input: 100
'''
Erdös asked whether the series
 sum n=1 to inf (−1)^n*n/p_n
converges.
Here, p_n denotes the n-th prime.
-1/2 + 2/3 - 3/5 + 4/7 - 5/11 + 6/13 - 7/17 + 8/19 - 9/23 + ...

See:
  Guy R.K. "Unsolved problems in number theory" 
  (2nd ed., Springer, 1994), page 203, E7 
'''

import matplotlib.pyplot as plt
import pUtils


n = int(input("n? "))
prims = pUtils.primesNum(n)
print(prims)

ys = []
xs = list(range(1, n+1))
tot = 0
for i in xs:
  tot += (pow(-1,i)*i)/prims[i-1]
  ys.append(tot)

plt.figure(figsize = (12, 5))
plt.plot(xs, ys)
plt.xlabel("n")
plt.ylabel("sum (−1)^n*n/p_n")
plt.title("Erdös' Convergence")

plt.show()


