
# erdosSub.py
# Andrew Davison, ad@coe.psu.ac.th, Nov. 2024
# David Wells - "Prime Numbers": Erdos, Paul
'''
There are infinitely many primes, p, such that every even number less than p - 2 can be expressed as the difference of two primes less than or equal to p. 

For example, if p = 23, 
2=19-17 4=17-13 6=17-11 8=19-11 10=17-7 12=19-7 
14=19-5 16=19-3 18=25-5 20=25-3
= 10 ways
'''

import pUtils

def hasPrimeDiff(evenNum, psSet):
  for prime in psSet:
    if (evenNum + prime) in psSet:
      return True
  return False


# generate primes where every even number < p-2 can be expressed as the difference of two primes
def findDiffPrimes(limit):
  primes = pUtils.primes(limit)
  diffPrims = []
  for p in primes:
    numMatches = 0
    psSet = set(pUtils.primes(p+1))
    # Check every even number less than p-2
    for evenNum in range(2, p-2, 2):
      if not hasPrimeDiff(evenNum, psSet):
        break
      numMatches += 1
    if numMatches > 0:
      diffPrims.append((p,numMatches))
  return diffPrims


limit = 100
diffPrims = findDiffPrimes(limit)
print("(prime p, no. of matches for all evens < p-2):")
print(diffPrims)
