
# euler808.py
# Project Euler problem 808: Reversible Prime Squares
# Andrew Davison, ad@coe.psu.ac.th, Nov. 2024
# https://projecteuler.net/problem=808
'''
Both 169 and 961 are the squares of a prime, and 
169 is the reverse of 961. We call such a number 
a reversible prime square if:
1. It is not a palindrome, and
2. It is the square of a prime, and
3. Its reverse is also the square of a prime.

169 and 961 are not palindromes, so both are 
reversible prime squares.

Find the sum of the first 50 reversible prime squares.

Answer: 3807504276997394

note: it takes around 3 - 8 secs to finish
'''

import time, math
import pUtils


def reverse(n):
  nStr = str(n)
  return int(nStr[::-1])

def isSquare(x):
  return (round(math.sqrt(x))**2 == x)


start = time.time()

limit = 5*(10**7)
prims = pUtils.primes(limit)
psSet = set(prims)

vals = []
for x in prims[4:]:
  sq = x*x
  rev = reverse(sq)
  if (not sq == rev) and isSquare(rev) and \
     (int(math.sqrt(rev)) in psSet):
    vals += [sq, rev]

revPs = sorted(set(vals))[:50]
print("First 50:", revPs)
print("Sum:", sum(revPs)) 

print(f"{(time.time() - start):.3f} secs")