
# eulerRecip.py
# Andrew Davison, ad@coe.psu.ac.th, Nov. 2024
# David Wells - "Prime Numbers": Euler and the reciprocals of the primes
# https://en.wikipedia.org/wiki/Divergence_of_the_sum_of_the_reciprocals_of_the_primes
'''
The alternating sum of the prime reciprocals converges, 
since its value always lies between 1/2 and O:   
1/2 -1/3 + 1/5 - 1/7 + 1/11 - 1/13 + ...
= 0.2696063519...
'''

from fractions import Fraction as F
import matplotlib.pyplot as plt
import pUtils

CONVERGENCE = 0.2696063519

limit = 200
prims = pUtils.primes(limit)
sumRs = []
tot = F(0,1)
for i, p in enumerate(prims):
  tot += F((-1)**i,p)   # Alternate sign for each term
  sumRs.append(float(tot))
print("last sum:", sumRs[-1])  # 0.2671458138820632

plt.plot(prims, sumRs)

plt.axhline(y=CONVERGENCE, color='red')
plt.text(100, CONVERGENCE-0.02,"y="+str(CONVERGENCE))

plt.title("Prime Reciprocals with Alternating Signs")
plt.xlabel("Prime Numbers $p_n$")
plt.ylabel(r'Alternating Series $\sum \frac{(-1)^n}{p_n}$')
plt.show()
