
# eulerTheorem.py
# https://en.wikipedia.org/wiki/Euler%27s_theorem
'''
Euler's primality test is an extension of Fermat's Little 
Theorem. It makes use of the Euler's Totient Function ϕ(n), 
which counts the number of integers less than n that are coprime with n. 

Euler's theorem states that for any integer a and n such 
that gcd⁡(a,n)=1, the following holds:
a^ϕ(n) ≡ 1 (mod n)

In the case when n is prime, ϕ(n)=n−1, so the test becomes 
similar to Fermat’s Little Theorem. However, Euler's test 
can also be applied to composite numbers.

Video:
"Introduction to Number Theory (Berkeley Math 115)
11. Euler's Theorem"
Richard E Borcherds
https://www.youtube.com/watch?v=V4cB7t-zHxE&list=PL8yHsr3EFj53L8sMbzIhhXSAOpuZ1Fov8&index=11
'''
# inputs:   91  or  97

import random
import pUtils


def phiTest(a, n):
  # Test Euler's Theorem for a given number n and base a.
  if n <= 1:
    return False
  if a % n == 0:
    return False
  pn = pUtils.phi(n)
  # is a^phi(n) ≡ 1 (mod n)
  return pow(a, pn, n) == 1


def isProbPrime(n, k=5):
  # use Euler's theorem as a probablistic prime tester
  if n == 2: 
    return True
  if n <= 1 or n % 2 == 0:
    return False
  # Perform test k times with random a values
  for _ in range(k):
    a = random.randint(2, n-1) 
    if not phiTest(a, n):
      return False 
  return True  # probably prime


p = int(input("p? "))
print(f"{p} prime? {isProbPrime(p)}")
