
# eulerTotNNPlot.py
# Andrew Davison, ad@coe.psu.ac.th, Nov. 2024
# David Wells - "Prime Numbers": Euler, Leonhard (1707-1783), Euler’s totient (phi) function
# https://en.wikipedia.org/wiki/Euler%27s_totient_function
'''
phi(n)/n < 1 for every integer n > 1. As n grows very
large, the average value of phi(n)/n approaches 6/pi^2. 
But the totient—ratio phi(n)/n itself does NOT tend 
towards its average — its structure is far more complex.
Also see eulerTotient.py.
'''

import math
import pUtils
import matplotlib.pyplot as plt

limit = 10000
xs = list(range(1, limit+1))
ys = [ pUtils.phi(x)/x for x in xs]

plt.figure(figsize=(10, 4))
plt.scatter(xs, ys, s=1)

avg = 6/(math.pi**2)
plt.axhline(y=avg, color='red')
plt.text(10000, avg-0.05,"y=6/$\pi^2$", color='red')

plt.title("$\phi(n)/n$")
plt.xlabel("n")
plt.ylabel('$\phi(n)/n$')
plt.show()
