
# eulerTotPlot.py
# Andrew Davison, ad@coe.psu.ac.th, Nov. 2024
# David Wells - "Prime Numbers": Euler, Leonhard (1707-1783), Euler’s totient (phi) function
# https://en.wikipedia.org/wiki/Euler%27s_totient_function
# Also see eulerTotient.py
'''
For a partial explanation of the lines, see:
https://math.stackexchange.com/questions/263243/lines-in-the-euler-phi-graph

'''

import pUtils
import matplotlib.pyplot as plt

def plotLine(plt, fn, label, col='red'):
  xs = [i for i in range(1, limit+1)]
  ys = [fn(x) for x in xs]
  plt.plot(xs, ys, color=col, alpha=0.4)
  plt.text(xs[-1], ys[-1], "y="+str(label), color=col)


def pf(x, q):
  return pUtils.phi(q)*(x/q - 1)


# plot φ(n) for n up to 1000
limit = 1000
ns = list(range(1, limit+1))
ys = [ pUtils.phi(n) for n in ns]
plt.scatter(ns, ys, s=2)

plotLine(plt, lambda x: x-1, "x-1", 'g')
plotLine(plt, lambda x: x/3-1, "x/3-1", 'g') 

plotLine(plt, lambda x: pf(x, 2), "pf(x,2)")
plotLine(plt, lambda x: pf(x, 3), "pf(x,3)")
plotLine(plt, lambda x: pf(x, 5), "pf(x,5)")
# plotLine(plt, lambda x: pf(x, 7), "pf(x,7)")

plt.title("Totient Function and lines")
plt.xlabel("n")
plt.ylabel('$\phi(n)$')
plt.show()
