
# fact37.py
# Andrew Davison, ad@coe.psu.ac.th, Nov. 2024
'''
37 is the median value for the second factor of an integer; 
thus the probability that the second factor of an integer 
chosen at random is smaller than 37 is approximately 1/2

Based on the blog post:
  "A truly incredible fact about the number 37"
  Chris Grossack
  https://grossack.site/2023/11/08/37-median.html
'''

import math, statistics
import pUtils

def getSecondSmallestFactors(limit):
  facts = []
  for i in range(2, limit):
    fs = pUtils.primeFactors(i)
    if len(fs) > 1:
      facts.append(fs[1])
    else:
      # If there's only one prime factor
      # say the second prime factor is infinity
      facts.append(math.inf)
  return facts


def lambda2(p, prims):
  """ Compute the density of the set of limit whose 
    second prime is p.
    See equation (1.3) in Koninck and Tenenbaum, 2002
  """
  s = 0
  out = 1
  for q in prims:
    if q >= p:
      break
    out *= (1 - (1/q))
  for q in prims:
    if q >= p:
      break
    s += (1/q) * 1/(1 - (1/q))
  out *= (s * 1/p)
  return out


limit = 100000
facts = getSecondSmallestFactors(limit)
med = statistics.median(facts)
print(f"Median second factor for primes < {limit}: {med}")

below37 = len([d for d in facts if d <= 37])/limit
print(f"Fraction whose second factor is <= 37: {below37:.4f}")

# -------------------------------------------

limit = 100
prims = pUtils.primes(limit)
total = 0
print(f"\nConsidering all primes < {limit} with total <= 0.5")
print("Prime p   2nd factor is p   total with 2nd factor <= p")
for p in prims:
  if total > 0.5: 
    break
  l = lambda2(p, prims)
  total += l
  print(f"  {p:3d}      {l:.4f}            {total:.4f}")
