
# feitThompson.py
# Andrew Davison, ad@coe.psu.ac.th, Nov. 2024
# David Wells - "Prime Numbers": Feit-Thompson conjecture
# https://en.wikipedia.org/wiki/Feit%E2%80%93Thompson_conjecture
'''
The Feit-Thompson conjecture asserts that there are 
no primes p and q for which (p^q-1)/(p-1) and 
(q^p-1)/(q-1) have a common factor.

However, the counterexample p = 17, q = 3313 with a common 
factor of 112,643 was found by Stephens. 
There are no other such pairs with both values less than 400,000.
'''

import pUtils

limit = 20
prims = pUtils.primes(limit)
nPs = len(prims)
facsFound = 0
print("Checking prime pairs up to", limit)
for i in range(nPs-1):
  for j in range(i+1, nPs):
    p = prims[i]
    q = prims[j]
    print(f"({p} {q}) ", end ='', flush=True)
    pFacs = pUtils.primeFactors((p**q - 1)//(p-1))
    qFacs = pUtils.primeFactors((q**p - 1)//(q-1))
    pqInt = set(pFacs).intersection( set(qFacs))
    if pqInt != set():
      print("**", pqInt)
      facsFound += 1
print()
print("Common factors found:", facsFound)
