
# fermat4n1.py
# Andrew Davison, ad@coe.psu.ac.th, Nov. 2024
# David Wells - "Prime Numbers": Fermat and primes of the form x^2 + y^2
# https://en.wikipedia.org/wiki/Fermat%27s_theorem_on_sums_of_two_squares
# https://mathworld.wolfram.com/Fermats4nPlus1Theorem.html

import math
import pUtils

def twoSquares(p):
  # can the prime be expressed as a sum of two squares
  for a in range(1, int(math.sqrt(p)) + 1):
    b2 = p - a**2
    b = math.isqrt(b2)
    if b**2 == b2:
      return a, b
  return None


limit = 100
print(f"Check Fermat's 4n+1 theorem for primes 5-{limit}:")
ps = pUtils.primes(5, limit+1)
for p in ps:
  if p%4 == 1:
    sqs = twoSquares(p)
    if sqs != None:
      a, b = sqs
      print(f"{p} = {a}^2 + {b}^2")
  
