
# fermatPTest.py
# Andrew Davison, ad@coe.psu.ac.th, Nov. 2024
# David Wells - "Prime Numbers": primality testing

# https://en.wikipedia.org/wiki/Fermat_primality_test
# https://en.wikipedia.org/wiki/Primality_test
# https://en.wikipedia.org/wiki/Mersenne_prime

'''
input: 2, 3, 5, 7, 13, 17, 19, 31, 61, 89, 107, 
       127, 521, 607, 1279, 2203, 2281, 3217, 
       4253, 9689, 9941 ...

  M_9941 timings: 7 secs, 9 secs
'''
 
import math, random, time
import pUtils
  

def isPrimeFermat(n, k=3):
  ''' If n is prime, then always returns true,
    If n is composite than returns false with
    high probability Higher value of k increases
    probability of correct result
  '''
  if n == 1 or n == 4:
    return False
  elif n == 2 or n == 3:
    return True
  else:
    for i in range(k):  # Try k times
      # Pick a random number in [2..n-2]  
      a = random.randint(2, n-2) 
      if pow(a, n-1, n) != 1:  # Fermat's little theorem
        return False
  return True



n = int(input("n of Mersenne M_n? "))
mn = (1<<n) - 1  # 2**n - 1

start = time.time()
print(isPrimeFermat(mn))
print(f"isPrimeFermat(): {(time.time()-start):.2f} secs")

start = time.time()
print(pUtils.isPrime(mn))
print(f"pUtils.isPrime(): {(time.time()-start):.2f} secs")
