
# fermatPrimes.py
# Andrew Davison, ad@coe.psu.ac.th, Nov. 2024
# David Wells - "Prime Numbers": Fermat's coniecture, Fermat numbers, and Fermat primes
# https://mathworld.wolfram.com/FermatPrime.html
'''
Generate Fermat primes of the form F_n = 2^(2^n)+1, where 
n is a non-negative integer. It's believed that there may 
be only five such primes.

First few Fermat numbers: n = 0 to 7
  3, 5, 17, 257, 65537, 4294967297, 
  18446744073709551617, 340282366920938463463374607431768211457, ...

The only known Fermat primes are the first five 
Fermat numbers: F_0=3, F_1=5, F_2=17, F_3=257, 
and F_4=65537.
'''

import pUtils

def fermatPrimes(limit):
  fnums = []
  for n in range(limit+1):
    fn = 2**(2**n) + 1
    if pUtils.isPrime(fn):
      fnums.append((n,fn))
  return fnums

limit = 7 
fPrims = fermatPrimes(limit)
print(f"Fermat primes in F_0 to F_{limit}:")
for (n, prim) in fPrims:
  print(f" F_{n}: {prim}")

