
# fibs.py
# Andrew Davison, ad@coe.psu.ac.th, Nov. 2024
# David Wells - "Prime Numbers": Fibonacci numbers
# https://en.wikipedia.org/wiki/Fibonacci_sequence
# input: 22 or 23 or 26
# Also see lucasSeq.py
'''
  functions for calculating the Fibonacci numbers.
  Start the sequence with 0, 1, for fib(0), fib(1)
'''

import math, random
import pUtils


PHI_F = (1 + math.sqrt(5)) / 2
  

def fib(n):
  a = 0; b = 1  # F0, F1
  if n <= 0:
    return a
  elif n == 1:
    return b
  else:
    for i in range(2, n+1):
      a, b = b, a + b
    return b


def isPower(num):
  if num < 1: 
    return False
  if num == 1:
    return True
  for base in range(2, int(math.sqrt(num)) + 1):
    exp = 2    # 2 or more
    while base**exp <= num:
      if base**exp == num:
        return True
      exp += 1
  return False


def isSumDistincts(n, fibs):
  ''' Zeckendorf's theorem: every positive integer can 
    be represented uniquely as the sum of one or more 
    distinct Fibonacci numbers in such a way that the sum 
    does not include any two consecutive Fibonacci numbers
    https://en.wikipedia.org/wiki/Zeckendorf%27s_theorem
  '''
  fs = []
  i = len(fibs) - 1
  while n > 0:
    if fibs[i] <= n:
      fs.append(fibs[i])
      n -= fibs[i]
      # Skip the next Fib to ensure non-consecutive use
      i -= 1
    i -= 1
  if n == 0:
    return True, fs
  else:
    return False, fs



print(f"\Fibs from f_0 to f_15:")
print(*[fib(n) for n in range(0, 16)])

# -----------------------------------

limit = 50
print(f"\nPrime Fibs from f_0 to f_{limit}")
for n in range(limit+1): 
  fn = fib(n)
  if pUtils.isPrime(fn):
    print(f"fib({n})= {fn}")

# -----------------------------------

print(f"\nPower Fibs from f_0 to f_{limit}")
for n in range(limit+1): 
  fn = fib(n)
  if isPower(fn):
    print(f"fib({n})= {fn}")

# -----------------------------------

max = 20
fibs = [ fib(n) for n in range(limit+1)]
print(f"\nWriting 1 to {max} as a sum of distinct non-consecutive fibs:")
for num in range(1, max+1):
  isDistinct, fs = isSumDistincts(num, fibs)
  if not isDistinct:
    print(num, "*")
  else:
    print(num, fs)


# -----------------------------------

n = random.randint(5, 30)
m = random.randint(5, 30)
fn = fib(n)
fm = fib(m)
print(f"\nRandomly using {n}, {m}")
print(f"f_{n} = {fn} and f_{m} = {fm}")
gcdIdx = math.gcd(n, m)
gcdFib = math.gcd(fn, fm)
print(f"GCD({n},{m}) = {gcdIdx} and GCD(f_{n}, f_{m}) = {gcdFib}")
print(f"and f_{gcdIdx} also = {fib(gcdIdx)}")
