
# formWilson.py
# Andrew Davison, ad@coe.psu.ac.th, Nov. 2024
# David Wells - "Prime Numbers": formulae for primes
# https://en.wikipedia.org/wiki/Formula_for_primes
# Also see wilson.py

'''
A simple prime formula:
    f(n) = floor((n! mod (n+1))/n)*(n−1) + 2 

By Wilson's theorem (see wilson.py), n+1 is prime iff 
n! ≡ n (mod n+1). Thus, when n+1 is prime, the 
first factor in the product is 1, and the formula 
produces the prime number n+1. But when n+1 is not prime, 
the first factor is 0 and the formula produces the 
prime number 2. 
'''

import math

def wilsonPrime(n):
  return math.floor((math.factorial(n) % (n+1))/n)*(n-1) + 2

limit = 100
print(f"Wilson generated primes up to {limit}:")
for i in range(2, limit):
  p = wilsonPrime(i)
  if p != 2:
    print(p, end=' ')
print()