
# fortunate.py
# Andrew Davison, ad@coe.psu.ac.th, Nov. 2024
# David Wells - "Prime Numbers": Fortunate numbers and Fortune's conjecture
# https://en.wikipedia.org/wiki/Fortunate_number
'''
A Fortunate number is the smallest integer m > 1 such that, 
for a given positive integer n, pn# + m is a prime number, 
where the primorial pn# is the product of the first n prime numbers. 
'''

import pUtils


def primorials(limit):  # from primorials.py
  primes = pUtils.primesNum(limit)
  prims = []
  # each primorial prim_n is the product of the 
  # first limit prime numbers. 
  prim = 1
  for p in primes:
    prim *= p
    prims.append(prim)
  return prims


def genFortunates(limit, ps):
  forts = []
  for n in range(0, limit):
    fn = fortunate_number(ps[n])
    forts.append((n, fn))
  return forts


def fortunate_number(prim):
  m = 2  # m > 1
  while not pUtils.isPrime(prim + m):
    m += 1
  return m


limit = 10  
ps = primorials(limit)
forts = genFortunates(limit, ps)
for n, fn in forts:
  print(f"F({n}) = {fn}")
