
# gammaPlot.py
# Andrew Davison, ad@coe.psu.ac.th, Nov. 2024
# info: https://en.wikipedia.org/wiki/Gamma_function
#       https://scipython.com/book/chapter-8-scipy/examples/the-gamma-function/
# Also see gammaL.py

import math
import matplotlib.pyplot as plt
from frange import *

def ngamma(x):
  try:
    return math.gamma(x)
  except ValueError:
    return math.inf

def ngamma2(x):
  if x == 0:
    return math.inf
  elif x > 0:
    return math.gamma(x)
  else:  # use Euler reflection
    return math.pi / (math.sin(math.pi*x) * math.gamma(1-x))


# some tests
print("ngamma(-5):", ngamma(-5))
print("ngamma(5):", ngamma(5))  # 4!
print("ngamma(0.5):", ngamma(0.5))  # sqrt(pi)
print("ngamma(-0.5):", ngamma(-0.5))
print("ngamma(-1.5):", ngamma(-1.5))

xs = linspace(-5, 5, 500)
ys = [ngamma(x) for x in xs]

plt.plot(xs, ys, color="red")

plt.axhline(y=0, color='k')
plt.axvline(x=0, color='k')
plt.title("Python's $\Gamma(x)$")
plt.xlabel("x")
plt.ylabel("$\Gamma(x)$")
plt.ylim(-4, 4)
plt.grid(True)
plt.show()