
# gapsBar.py
# Andrew Davison, ad@coe.psu.ac.th, Nov. 2024
# David Wells - "Prime Numbers": gaps between primes and composite runs
# https://en.wikipedia.org/wiki/Prime_gap
# Also see gapsPlot.py, gaps.py, gapsCramer.py

from collections import Counter
import pUtils
import matplotlib.pyplot as plt


limit = 1000000
primes = pUtils.primes(limit)

primes = pUtils.primes(10**6)
gaps = [primes[i+1] - primes[i] for i in range(len(primes)-1)]
gapCounts = Counter(gaps)

gapVals = list(gapCounts.keys())
freqs = list(gapCounts.values())

plt.figure(figsize=(10, 4))
bars = plt.bar(gapVals, freqs)

for gap, bar, freq in zip(gapVals, bars, freqs):
  if freq < 2000 and gap%10 == 0:
    plt.text(bar.get_x() + bar.get_width()/2, bar.get_height(),
           str(freq), ha='center', va='bottom')


plt.xlabel('Prime Gap')
plt.xticks(range(0, gapVals[-1]+10, 10))
plt.ylabel('Frequency')
plt.title('Frequency of Gaps Between Primes Up to One Million')
plt.show()
