
# gapsCramer.py
# Andrew Davison, ad@coe.psu.ac.th, Nov. 2024
# David Wells - "Prime Numbers": gaps between primes and composite runs
# https://en.wikipedia.org/wiki/Cram%C3%A9r%27s_conjecture
# https://en.wikipedia.org/wiki/Prime_gap
# Also see gaps.py, gapsBar.py, gapsPlot.py

'''
Cramér's prime conjecture is a number theory conjecture 
proposed by Harald Cramér in 1936. It provides an estimate 
for the gaps between consecutive prime numbers. 

Cramér suggested that the gap between consecutive primes 
p_n​ and p_n+1​ grows roughly like
(log p_n)^2, meaning
  p_n+1 − p_n = O((log p_n)^2)
A more specific version of this conjecture predicts that:
  p_n+1 − p_n < C(log ⁡p_n)^2
for some constant C.
'''

import math
import pUtils


limit = 100000
prims = pUtils.primes(limit)

# Calculate 20 prime gaps and compare with (log p_n)^2
print("Prime   gap  log^2   gap < log^2")
for i in range(1, 21):  # first 20
  p_n = prims[i - 1]
  p_next = prims[i]
  gap = p_next - p_n
  logSqr = (math.log(p_n) ** 2)
  print(f"  {p_n:2d}     {gap}  {logSqr:6.2f}    {gap < logSqr}")
