
# gapsPlot.py
# Andrew Davison, ad@coe.psu.ac.th, Nov. 2024
# David Wells - "Prime Numbers": gaps between primes and composite runs
# https://en.wikipedia.org/wiki/Prime_gap
# Also see gaps.py, gapsBar.py, gapsCramer.py

import math
import pUtils
import matplotlib.pyplot as plt

numPs = 1000
primes = pUtils.primesNum(numPs)

# Calculate the gaps between consecutive primes
gaps = []
for i in range(1, numPs):
  gaps.append(primes[i] - primes[i-1])

# Compute log(N) for each prime
logPs = [ math.log(p) for p in primes[:-1]]  
         # excluding last prime as gap is n-1


plt.figure(figsize=(10, 6))
xs = primes[:-1]

plt.plot(xs, gaps, label="Prime Gap", 
     marker='o', markersize=3, linestyle='-', color='b')
plt.plot(xs, logPs, label="log(N)", 
                 linestyle='--', color='r')

plt.title('Prime Gaps vs log(N)')
plt.xlabel('Prime N')
plt.ylabel('Gap')
plt.legend()
plt.grid(True)
plt.show()
