
# gaussianPrimes.py
# Andrew Davison, ad@coe.psu.ac.th, Nov. 2024
# David Wells - "Prime Numbers": Gaussian primes
# https://mathworld.wolfram.com/GaussianPrime.html
# https://en.wikipedia.org/wiki/Gaussian_integer

import math
import pUtils
import matplotlib.pyplot as plt
from matplotlib.patches import Circle


def genGaussians(limit):
  # generate Gaussian primes
  gPrims = []
  for a in range(-limit, limit+1):
    for b in range(-limit, limit+1):
      if pUtils.isGaussianPrime(a, b) and \
        ((a**2 + b**2) <= limit**2):
        gPrims.append((a, b))  # store as tuples
  return gPrims



ptsCnts = [len(genGaussians(r)) for r in range(10)]
print("First 10 Gaussian primes counts:", ptsCnts)

# ---------------------------
limit = 32
gPrims = genGaussians(limit)
xs, ys = zip(*gPrims) 

plt.figure(figsize=(8, 8))
ax = plt.gca()
plt.scatter(xs, ys, color='black', s=40, marker=',')
circle = Circle((0,0), limit, lw=2, color='red', fill=False)
ax.add_patch(circle)
ax.set_aspect('equal')
plt.title(f'Gaussian Primes in (-{limit}, {limit})')
plt.axis('off')
plt.show()
