
# gaussianSpiral.py
# Andrew Davison, ad@coe.psu.ac.th, Nov. 2024
# https://mathoverflow.net/questions/91423/gaussian-prime-spirals
# https://scipython.com/book2/chapter-3-simple-plotting/problems/p32/the-gaussian-prime-spiral/
# inputs:  3 5   5 23    8 13   -12 -7   27 30   127 130

'''
Consider the sequence of Gaussian integers traced out 
by a particle, initially at (x0,y0), moving in the complex 
plane according to the following rule: it takes integer 
steps in its current direction (+/- 1 in either the real 
or imaginary direction) but turns left after it encounters 
a Gaussian prime. 

Its initial direction is the positive real direction. 
The path traced out by the particle is called a Gaussian 
prime spiral. 

So far, no matter what starting point people have used, 
the iteration forms a closed loop, returning to its starting 
point. But no one has been able to prove that this always happens.
'''

import matplotlib.pyplot as plt
import pUtils


def turnLeft(dx, dy):
  # cycle through (1,0)->(0,1)->(-1,0)->(0,-1)->(1,0)
  return -dy, dx


# Starting point and direction
x0, y0 = map(int, input("x0 y0? ").split())
x = x0; y = y0
dx, dy = 1, 0

# track the path taken
xPath, yPath = [x], [y]

max_steps = 10000
step = 0
while step < max_steps:
  step += 1
  x, y = x+dx, y+dy
  xPath.append(x)
  yPath.append(y)
  if (x,y) == (x0,y0):
    print(f'Returned to ({x}, {y}) in {step} steps.')
    break
  if pUtils.isGaussianPrime(x,y):  # stored as a tuple
    dx, dy = turnLeft(dx,dy)
else:
  print(f'max steps = {max_steps} reached.')


plt.plot(xPath, yPath, lw=2)
plt.title('Gaussian Prime Spiral starting at ' + str((x0,y0)))
plt.show()