
# gilbreath.py
# Andrew Davison, ad@coe.psu.ac.th, Nov. 2024
# David Wells - "Prime Numbers": Gilbreath’s conjecture
# https://en.wikipedia.org/wiki/Gilbreath%27s_conjecture
# input: 72  or 52
'''
Gilbreath's Conjecture relates to the first differences 
of the sequence of prime numbers:

* Start with the sequence of prime numbers p1,p2,p3,...

* Create a new sequence by taking the absolute differences 
  between consecutive primes: p2−p1,p3−p2,p4−p3,...

* Repeat this process iteratively on the new sequence

Note that the first term of each new sequence will always 
be 1.
'''

import math
import pUtils


def genDiffs(seq):
  return [abs(seq[i+1] - seq[i]) 
                     for i in range(len(seq) - 1)]


n = int(input("n? "))
ps = pUtils.primes(n)
seq = ps
print(*seq)
count = 0
while len(seq) > 1:
  seq = genDiffs(seq)
  print(count*' ', *seq)
  count += 1
