
# giuga.py
# Andrew Davison, ad@coe.psu.ac.th, Nov. 2024
# David Wells - "Prime Numbers": Giuga’s conjecture
# https://mathworld.wolfram.com/GiugasConjecture.html
'''
Giuga's prime conjecture states that a number n is prime 
if and only if:
n−1 | sum k=1 to n−1 k^(n−1)

That is, n−1 divides the sum of the powers of integers 
from 1 to n−1, each raised to the (n−1)-th power. This 
conjecture remains unproven but is valid for all 
known primes.
'''

def isGiugaPrime(n):
  if n <= 1:
    return False
  sumPows = sum(pow(k, n-1, n) for k in range(1, n))
  return (sumPows % n == n-1)


limit = 100
for n in range(2, limit+1):
  if isGiugaPrime(n):
    print(n, end=' ')
print()