
# giugaNums.py
# Andrew Davison, ad@coe.psu.ac.th, Nov. 2024
# David Wells - "Prime Numbers": Giuga’s numbers
# https://mathworld.wolfram.com/GiugaNumber.html
'''
If n is a composite number such that p | n/p - 1 for 
all prime divisors, p, of n, then n is a Giuga number. 
Giuga proved that n is a Giuga number if and only if 
the sum of the reciprocals of its prime divisors, 
less their product, is an integer.
'''

import pUtils
from fractions import Fraction as F

def isGiugaNumber(n):
  if pUtils.isPrime(n):
    return False  # Giuga numbers must be composite

  divs = pUtils.primeFactors(n)
  for p in divs:
    if (n//p - 1) % p != 0:
      return False

  # Check the reciprocals sum condition
  gc = sum([F(1, d) for d in divs]) - F(1,n)
  return gc == F(1,1)


limit = 2000
for n in range(2, limit + 1):
  if isGiugaNumber(n):
    print(n, pUtils.primeFactors(n))

