
# jacobiSymbol.py
# https://en.wikipedia.org/wiki/Jacobi_symbol
# https://en.wikipedia.org/wiki/Legendre_symbol

# inputs:  10 13  or   12345 331  or  1001 9907
'''
The Jacobi symbol (k|n) is k generalization of the 
Legendre symbol that is defined for any integer k
and positive odd integer n (not necessarily prime). 
The Jacobi symbol can be 
computed using k recursive algorithm based on the 
properties of quadratic reciprocity.

This generalization provides an efficient way to 
compute all Legendre symbols without performing 
factorization along the way.

See quadRecip.py
'''

import pUtils


def jacobiSymbol(k, n):   # (k | n)
  if n % 2 == 0:
    raise ValueError("Only defined for odd n")
  # Make sure k is positive and reduce modulo n
  k = k % n
  result = 1
  while k != 0:
    # Handle the factor of 2 in k
    while k % 2 == 0:
      k //= 2
      if n % 8 in [3, 5]:
        result = -result
    # Swap k and n using the law of quadratic reciprocity
    k, n = n, k
    if k % 4 == 3 and n % 4 == 3:
      result = -result
    k = k % n
  if n == 1:
    return result
  else:
    return 0


'''
k, n = map(int, input("k n? ").split())
print(f"Jacobi symbol ({k}|{n}) = {jacobiSymbol(k, n)}")
'''

ks = range(0, 15)
ns = range(1, 19, 2)
print("Jacobi Symbol (k|n):")
header = "n\k " + " ".join(f"{col:>2}"  for col in ks)
print(header)
for n in ns:
  row = [jacobiSymbol(k, n) for k in ks]
  print(f"{n:>2}  " + " ".join(f"{num:>2}" for num in row))

