
# legendre.py
# Andrew Davison, ad@coe.psu.ac.th, Nov. 2024
# David Wells - "Prime Numbers": Landau, Edmund (prob 4)
# https://en.wikipedia.org/wiki/Legendre%27s_conjecture
# Also see landau4.py
'''
Legendre's Prime Conjecture states that there is always 
at least one prime number between any two consecutive 
perfect squares. For every integer n:
  n^2 < p < (n+1)^2
where p is a prime number
'''

import math
import pUtils
import matplotlib.pyplot as plt


limit = 1000
nps = []  
for n in range(limit+1):
  prims = pUtils.primes(n**2, (n+1)**2+1)
  nps.append(len(prims))

plt.scatter(range(limit+1), nps, s=2, label="Legendre")

approxs = [ n/math.log(n) for n in range(2, limit+1)]
plt.plot(range(2,limit+1), approxs, 
                  label=r'$\frac{n}{\log(n)}$', 
                  linestyle='--', color='red', lw=2)

plt.xlabel("n")
plt.ylabel("Primes count")
plt.title("No. of primes between n^2 and (n+1)^2")
plt.legend()
plt.show()

