
# legendreConst.py
# Andrew Davison, ad@coe.psu.ac.th, Nov. 2024
# David Wells - "Prime Numbers": Legendre, A. M. (1752-1833)
# https://en.wikipedia.org/wiki/Legendre%27s_constant
# https://mathworld.wolfram.com/LegendresConstant.html
'''
Legendre hypothesized that the number of primes <= n 
could be approximated by:
  π(n) ≈ n /(log⁡(n)− A)
where π(n) is the prime-counting function, and A is 
around 1.08366, now called Legendre's constant.
'''

import math
import pUtils
import matplotlib.pyplot as plt

A = 1.08366   # Legendre's constant


def legendreEst(n):
  if n < 2:
    return 0
  else:
    return n/(math.log(n) - A)


limit = 1000 
xs = range(2, limit+1)
pis = [ pUtils.pi(n) for n in xs]
lpis = [ legendreEst(n) for n in xs]

plt.plot(xs, pis, label=r'$\pi(n)$', color='blue')
plt.plot(xs, lpis, label=r'$\frac{n}{\ln(n) - A}$', 
                  linestyle='--', color='red')

plt.title(f'$\pi(n)$ and Legendre\'s Est. with A = {A}')
plt.xlabel('n')
plt.ylabel('Prime Count')
plt.legend()
plt.grid(True)
plt.show()
