
# lehmanFact.py
# Andrew Davison, ad@coe.psu.ac.th, Nov. 2024
# https://second.wiki/wiki/faktorisierungsmethode_von_lehman
'''
Given an integer n > 21 either return
a single factor of n or report that n is prime. 
'''

import random, math
import pUtils

def lehman(n):
  k = 1
  np = n**(1/3)
  # Check for small prime factors first
  while pUtils.prime(k) < np:
    if math.gcd(pUtils.prime(k), n) != 1:
      return pUtils.prime(k), False
    k += 1

  # switch to large factors using the Lehman method
  for k in range(1, math.ceil(np)+1):
    kn = 2 * math.sqrt(k*n)
    start = math.ceil(kn)
    end = math.floor(kn + \
               n**(1/6)/(4 * math.sqrt(k)) )
    for a in range(start, end+1):
      b2 = a**2 - 4*k*n
      b = math.isqrt(b2)
      if b*b == b2:  # is b a perfect square?
        return math.gcd(a+b, n), False
  return n, True


n = random.randint(22, 2**20)
print("random n =", n)
f, isPrime = lehman(n)
print("Factor:", f)
print("n is prime?", isPrime)
print("pUtils factorization:", pUtils.factorizeDict(n))

