
# lemoine.py
# Andrew Davison, ad@coe.psu.ac.th, Nov. 2024
# https://en.wikipedia.org/wiki/Lemoine%27s_conjecture

'''
Lemoine's Prime Conjecture states that every odd integer greater than 5 can be expressed as the sum of a prime number and twice a prime number. In mathematical terms, for every odd integer n>5:
n=p+2q
Where p and q are prime numbers.
'''

import math
import pUtils

def lemoine_conjecture(limit):
  ps = pUtils.primes(limit)
  prime_set = set(ps)  # To check for primality quickly
  exprs = []

  # Check odd integers greater than 5
  for n in range(7, limit+1, 2):
    found = False
    for p in ps:
      if p >= n:
        break
      q = (n - p) // 2
      if (n - p) % 2 == 0 and q in prime_set:
        exprs.append((n, p, q))
        found = True
        break
    if not found:
      print(f"Lemoine's conjecture failed for n = {n}")
      return False
  return exprs


limit = 100 
results = lemoine_conjecture(limit)
if isinstance(results, list):
  print(f"Lemoine's conjecture holds for odd integers up to {limit}:")
  for n, p, q in results:
    print(f"{n} = {p} + 2*{q}")
else:
  print("Lemoine's conjecture does not hold for some odd integers.")
