
# lenstra.py
# Andrew Davison, ad@coe.psu.ac.th, Nov. 2024
# https://en.wikipedia.org/wiki/Mersenne_conjectures#Lenstra%E2%80%93Pomerance%E2%80%93Wagstaff_conjecture
# Also see mersenne.py
# input:  130
'''
The Lenstra–Pomerance–Wagstaff (LPW) conjecture proposes 
an estimate for the number of Mersenne primes less than 
a given bound n. It suggests that the number of Mersenne 
primes with an exponent less than n is asymptotically 
given by:
  E(n)=e^γ log2 ⁡⁡n
where γ is the Euler-Mascheroni constant.
'''

import pUtils
import math


def LPWEstimate(n):
  return math.exp(pUtils.GAMMA) * math.log2(n)


def mersennePrimes(limit):
  mprims = []
  for n in range(2, limit+1):
    val = (1 << n) - 1  # Equivalent to 2**n - 1
    if pUtils.isPrimeMR(val):  # was isPrime()
      mprims.append((n, val))
  return mprims


n = int(input("n for M_n? "))
mPrims = mersennePrimes(n)
lpw = LPWEstimate(n)

print(f"No. of Mersenne primes M_n for n < {n}: {len(mPrims)}")
print(f"LPW est.: {lpw:.2f}")
print("Mersenne primes (n, M_n):")
print(mPrims)
