
# li.py
# Andrew Davison, ad@coe.psu.ac.th, Nov. 2024
# David Wells - "Prime Numbers": prime number theorem and the prime counting function
# Used in primeCountsLi.py, primeCountsRi.py, primePlotLi.py
import math

def logReciprocal(t):
  return 1 / math.log(t) if t > 1 else 0  # Avoid div by zero


def li(x, n=10000):
  # Logarithmic integral approx using trapezoidal integration
  # https://en.wikipedia.org/wiki/Logarithmic_integral_function
  if x < 2:
    return 0  # undefined for x < 2
  return int( math.ceil( trapInt(logReciprocal, 2, x, n) ))


def trapInt(f, a, b, n):
  h = (b - a) / n
  integral = 0.5 * (f(a) + f(b))  # First and last terms
  for i in range(1, n):
    integral += f(a + i * h)
  return integral * h
