
# liPlot.py
# Andrew Davison, ad@coe.psu.ac.th, Nov. 2024
# https://en.wikipedia.org/wiki/Prime-counting_function
# David Wells - "Prime Numbers": Littlewood's theorem, prime number theorem and the prime counting function
# Also see primeCountsLi.py

import math
import matplotlib.pyplot as plt
import pUtils
from li import li

limit = 100
xs = list(range(2, limit+1))
pis = [ pUtils.pi(x) for x in xs]
approxs = [ x/math.log(x) for x in xs]
lis = [ li(x) for x in xs]

plt.figure(figsize=(10, 6))

plt.scatter(xs, pis, s=5, label="π(x)")
plt.plot(xs, approxs, label=r'$\frac{x}{\log(x)}$ approx', linestyle='--', color='red', lw=2)
plt.plot(xs, lis, label='li(x)', linestyle=':', color='green', lw=2)

plt.xlabel('x', fontsize=12)
plt.ylabel('π(x), li(x) and Approx', fontsize=12)
plt.title('π(x), li(x) and Approx $\\frac{x}{\log(x)}$', fontsize=14)
plt.grid(True)
plt.legend()

plt.show()
