
# littlewoodDiff.py
# Andrew Davison, ad@coe.psu.ac.th, Nov. 2024
# David Wells - "Prime Numbers": Littlewood's theorem, the prime numbers race
# https://www.math.stonybrook.edu/~moira/mat331-spr10/papers/2006%20GranvillePrime%20Number%20Races.pdf
'''
The prime number race between primes of the form 
4n+1 and 4n+3.
Also see littlewoodRace.py and bias.py
'''

import math
import matplotlib.pyplot as plt
import pUtils


limit = 700000
ns = list(range(2, limit+1))

# record diffs in 4n+1 and 4n+3 primes up to limit
count4n1 = 0 
count4n3 = 0 
pDiffs = [] 
for n in ns:
  if pUtils.isPrime(n):
    if n%4 == 1:
      count4n1 += 1
    elif n%4 == 3:
      count4n3 += 1
  pDiffs.append(count4n1 - count4n3)


plt.plot(ns, pDiffs)

plt.xlabel('n')
plt.ylabel('Count')
plt.title('Difference Count in Primes: (4n+1)-(4n+3)')
plt.axhline(y=0, color='k')
plt.grid(True)
plt.show()
