
# littlewoodRace.py
# Andrew Davison, ad@coe.psu.ac.th, Nov. 2024
# David Wells - "Prime Numbers": Littlewood's theorem, the prime numbers race
# https://www.math.stonybrook.edu/~moira/mat331-spr10/papers/2006%20GranvillePrime%20Number%20Races.pdf
'''
The prime number race between primes of the form 
4n+1 and 4n+3.
Also see littlewoodDiff.py and bias.py
'''

import matplotlib.pyplot as plt
import pUtils


limit = 1000
ns = list(range(2, limit+1))

# count primes of the form 4n+1 and 4n+3 up to limit
count4n1 = 0 
count4n3 = 0 
primes4n1 = [] 
primes4n3 = []
for n in ns:
  if pUtils.isPrime(n):
    if n%4 == 1:
      count4n1 += 1
    elif n%4 == 3:
      count4n3 += 1
    if count4n1 == count4n3:
      print("Tied at", n)
  primes4n1.append(count4n1)
  primes4n3.append(count4n3)


plt.plot(ns, primes4n1, label="4n+1 Primes", lw=2)
plt.plot(ns, primes4n3, label="4n+3 Primes", lw=2)

plt.xlabel('n')
plt.ylabel('Primes Count')
plt.title('Prime Number Race')
plt.grid(True)
plt.legend()
plt.show()
