
# longPrime.py
# Andrew Davison, ad@coe.psu.ac.th, Nov. 2024
# https://rosettacode.org/wiki/Long_primes
# https://en.wikipedia.org/wiki/Full_reptend_prime
'''
The long primes are those for which the period of 
1/p has length p-1. To base 10 these are  
7, 17, 19, 23, 29, 47, 59, 61, 97, 109, . . .

It seems that about 37 percent of the primes are long 
in base 10. Emil Artin suggested that this number is 0.3759558136... (see
https://en.wikipedia.org/wiki/Artin%27s_conjecture_on_primitive_roots)
'''

import pUtils

def isLongPrime(p):
  if not pUtils.isPrime(p):
    return False
  # Check if 10^(p-1) ≡ 1 (mod p) and no smaller exponent satisfies this
  if pow(10, p-1, p) != 1:
    return False
  # Check all divisors of p-1
  for q in range(1, p-1):
    if pow(10, q, p) == 1:
      return False
  return True


limit = 200
for p in range(2, limit):
  if isLongPrime(p):
     print(p, end=' ')
print()

print("Long primes % < limit")
for limit in [1000, 5000, 10000, 20000]:
  numLPs = sum([isLongPrime(i) for i in range(2, limit)])
  print(f"{limit:6d} : {(numLPs/pUtils.pi(limit)):.2%}")
     # seems to be around 38% ?