
# lucasPrime.py
# Andrew Davison, ad@coe.psu.ac.th, Nov. 2024
# David Wells - "Prime Numbers": Lucas, Edouard (1842-1891), primality testing
# https://en.wikipedia.org/wiki/Lucas_primality_test
# https://www.geeksforgeeks.org/lucas-primality-test/
# input: 2305843009213693951  or  
#        170141183460469231731687303715884105727  (Mersenne primes)
'''
A positive number n is prime if there exists an 
integer a (1 < a < n) such that: 
  a^(n-1) ≡ 1 (mod n)
And for every prime factor q of (n-1),
  a^((n-1)/q) not ≡ 1 (mod n)
'''

import random
import math
import pUtils
  
def isLucasPrime(n, numTries=3):
  if (n == 1):
    return False
  if (n == 2):
    return True
  if (n % 2 == 0):
    return False
    
  factors = pUtils.primeFactors(n-1)
  # print("factors", n-1, factors)

  for i in range(numTries):
    a = random.randint(2, n-1) 
    if (pow(a, n-1, n) != 1):
      return False

    # check that every factor 
    # of n-1 satisfy the condition
    allFacs = True
    for k in range(len(factors)):
      # If a^((n-1)/q) mod n equal 1
      if (pow(a, (n-1)//factors[k], n) == 1):
        allFacs = False
        break
    if (allFacs):
      return True
  return False  # probably composite
  

n = int(input("n? "))
print(isLucasPrime(n))

