
# luckyHawkins.py
# Andrew Davison, ad@coe.psu.ac.th, Nov. 2024
# David Wells - "Prime Numbers": lucky numbers, "random" primes

# https://demonstrations.wolfram.com/HawkinsNumbersAndThePrimeNumberTheorem/
'''
Hawkins random primes are randomly chosen from the 
numbers greater than the previous prime, with each number 
having a probability of being selected proportional to 
1/log⁡(n), which mimics the distribution of prime numbers in 
the prime number theorem.
'''

import random, math
import matplotlib.pyplot as plt
import pUtils


def hawkinsPrimes(limit):
  nums = list(range(2, limit+1)) 
  idx = 0
  while idx < len(nums)-1:
    prob = 1/nums[idx]
    saved = nums[:(idx+1)].copy()
    for i in range(idx+1, len(nums)):
       if random.random() > prob: # keep the number
         saved.append(nums[i])
    nums = saved
    idx += 1
    # print(idx, nums)
  return nums


def hCounts(hPrims):
  # no. of Hawkins primes <= 2..maxP
  maxP = hPs[-1]
  hcs = []
  for i in range(2, maxP+1):
    hcs.append( hCount(i, hPrims))
  return hcs


def hCount(n, hPrims):
  return sum(hp <= n for hp in hPrims)



limit = 1000
hPs = hawkinsPrimes(limit)
print("First Hawkins random primes:", hPs[:10])

print("No. of Hawkins random primes:", len(hPs))
maxHP = hPs[-1]
print("Largest Hawkins prime:", maxHP)

ps = pUtils.primes(limit)
print("\nNo. of primes:", len(ps))
print("Largest prime:", ps[-1])

hcs = hCounts(hPs)
ns = list(range(2, maxHP+1))
approxs = [ n/math.log(n) for n in ns]

plt.scatter(ns, hcs, s=5, label="Hawkins count")
plt.plot(ns, approxs, label=r'$\frac{n}{\log(n)}$', 
                  linestyle='--', color='red', lw=2)
plt.xlabel('n')
plt.title('Hawkins count and $\\frac{n}{\log(n)}$')
plt.grid(True)
plt.legend()
plt.show()

