
# modmult.py
# Andrew Davison, ad@coe.psu.ac.th, Nov. 2024
# http://inversed.ru/Blog_1.htm
# inputs: 257 155  or   257 132  or 255 5
'''
  Modular arithmetic visualization as a circle of
  numbers (1 to mod-1) connected by lines from node
  i to node d when d = (i*mult) % mod

  The circles and lines are drawn with graphviz
  (pip install graphviz)
  https://graphviz.readthedocs.io/en/stable/manual.html

  See the video:
  Times Tables, Mandelbrot and the Heart of Mathematics 
  Mathologer, https://www.youtube.com/watch?v=qhbuKbxJsk8&t=2s

  Graphviz is also used in drawPrimeSums.py

  Debugging:
    neato -Tpng modmult.gv > modmult.png
'''

import math
import graphviz

RADIUS = 2

def positionNodes(g, num):
  g.node_attr['shape']='point'
  g.node_attr['width']='0.01'
  g.node_attr['height']='0.01'
  angleIncr = math.pi*2/num
  for i in range(num):
    x = RADIUS*math.cos(math.pi/2 + angleIncr*i)
    y = RADIUS*math.sin(math.pi/2 + angleIncr*i)
    g.node(str(i), pos=str(x)+","+str(y)+"!")



mod, mult = map(int, input("mod mult? ").split())

g = graphviz.Graph('G', engine='neato', 
                           filename='modmult.gv')
g.graph_attr['label']="Points/mod: " + str(mod) + \
                    "; multiple: " + str(mult)
g.edge_attr['penwidth']='0.5'
positionNodes(g, mod)  

for i in range(1, mod): # skip loop at 0
  dest = (i*mult)%mod
  # print(i, "-->", dest)
  g.edge(str(i), str(dest), color='blue')

g.view()
