
# opperman.py
# Andrew Davison, ad@coe.psu.ac.th, Nov. 2024
# David Wells - "Prime Numbers": Opperman’s conjecture
# https://en.wikipedia.org/wiki/Oppermann's_conjecture
# input: 9
'''
The conjecture states that, for every integer n > 1, 
there is at least one prime number between
    n(n − 1) and n^2,
and at least another prime between
    n^2 and n(n + 1)

Or equivalently:
    π(n^2 − n) < π(n^2) < π(n^2 + n) for n > 1
'''

import pUtils
import matplotlib.pyplot as plt


limit = 30
ns = range(2, limit+1)
pisLess = [ pUtils.pi(n*n-n) for n in ns]
pis = [ pUtils.pi(n*n) for n in ns]
pisMore = [ pUtils.pi(n*n+n) for n in ns]

plt.figure(figsize=(6, 7))

plt.scatter(ns, pisLess, label='$\pi(n^2-n)$', s=6)
plt.scatter(ns, pis, label='$\pi(n^2)$', s=6)
plt.scatter(ns, pisMore, label='$\pi(n^2+n)$', s=6)

plt.xlabel('n')
plt.ylabel('Prime Counts')
plt.title("Opperman's Conjecture")
plt.legend()
plt.show()


