
# pascal.py
# Andrew Davison, ad@coe.psu.ac.th, Nov. 2024
# David Wells - "Prime Numbers": Pascal’s triangle and the binomial coefficients
# input:  5   or  11

import math


def genPascal(n):
  # generate Pascal's triangle up to n rows
  tri = [[1]]  # first row
  for i in range(1, n+1):
    row = [1]
    for j in range(1, i):
      # Each element (except the first and last) 
      # is the sum of two elements above it
      row.append(tri[i-1][j-1] + tri[i-1][j])
    row.append(1)
    tri.append(row)
  return tri


def formatList(nums):
  return ' '.join(f"{num:3}" for num in nums)
    # only looks nice up to row 12

def binoms(n):
  return [math.comb(n, k) for k in range(n+1)]



n = int(input("No. of rows? ")) 
tri = genPascal(n)
maxWidth = len(formatList(tri[-1]))
print()
for row in tri:
  print(formatList(row).center(maxWidth))  # print centered


print(f"\nBinomial coefs. for row {n}:", *binoms(n))


print("\nSierpinski's Gasket using the triangle:\n")
gasket = []
for row in tri:
  gRow = [("*" if n%2==1 else " ") for n in row]
  gasket.append(" ".join(gRow))

maxWidth = len(gasket[-1])
for row in gasket:
  print(row.center(maxWidth))


