
# perfect.py
# Andrew Davison, ad@coe.psu.ac.th, Nov. 2024
# David Wells - "Prime Numbers": abundant, deficient and perfect number
# https://en.wikipedia.org/wiki/Perfect_number

from fractions import Fraction as F
import pUtils


def isAbundant(n):
  pdNum = sum( pUtils.divisors(n, isProper=True))
  return (pdNum > n)

def isDeficient(n):
  pdNum = sum( pUtils.divisors(n, isProper=True))
  return (pdNum < n)

def isPerfect(n):
  pdNum = sum( pUtils.divisors(n, isProper=True))
  return (pdNum == n)


def genPerfects(limit):
  # Generate perfect numbers using Euclid's method
  pNums = []
  for p in range(2, limit):
    if pUtils.isPrime(p):
      mersennePrime = (2**p)-1
      if pUtils.isPrime(mersennePrime):
        pNum = (2**(p-1)) * mersennePrime
        pNums.append(pNum)
  return pNums


def genPerfectsSigma(limit):
  # Generate perfect numbers using the sigma function
  pNums = []
  for p in range(2, limit):
    if pUtils.sigma(p) == 2*p:
      pNums.append(p)
  return pNums


# --------------------------------------

if __name__ == "__main__":
  abundants = 0
  deficients = 0
  perfects = 0
  for n in range(1, 20001):
    pdNum = sum( pUtils.divisors(n, isProper=True))
    if pdNum == n:
      perfects += 1
    elif pdNum > n:
      abundants += 1
    else:
      deficients += 1
  
  print("No. of abundants, deficients, and perfects up to 20000:")
  print(abundants, deficients, perfects)
  
  
  print("\nAbundants up to 100:")
  for n in range(1, 100):
    if isAbundant(n):
      print(n, end=' ')
  print()
  
  print("\nDeficients up to 100:")
  for n in range(1, 100):
    if isDeficient(n):
      print(n, end=' ')
  print()
  
  print("\nPerfects up to 100:")
  for n in range(1, 100):
    if isPerfect(n):
      print(n, end=' ')
  print()
  
  print("\nEuclid perfects:", *genPerfects(30))
  
  
  print("\nOdd abundants up to 10000:")
  abunds = []
  for n in range(1, 10001):
    pdNum = sum( pUtils.divisors(n, isProper=True))
    if (pdNum > n) and (n%2 != 0):  # odd abundant
      abunds.append(n)
  print("No. found:", len(abunds))  # 23
  print(*abunds)
  
  # https://en.wikipedia.org/wiki/Superabundant_number
  print("\nSuper abundants:")
  superAbun = F(0,1)
  for n in range(1, 51):
    sa = F(pUtils.sigma(n), n)
    if sa > superAbun:
      print(f"sigma({n})/{n} =", sa)
      superAbun = sa


