
# pollardRho.py
# Andrew Davison, ad@coe.psu.ac.th, Nov. 2024
# https://en.wikipedia.org/wiki/Pollard%27s_rho_algorithm
'''
Given a composite number n, find a nontrivial factor.  
'''

import random, math
import pUtils

def pollard(n):
  while True:
    seed = random.randint(0, n-1)
    a = random.randint(0, n-1)
    x = seed; y = seed
    d = 1
    while d == 1:
      x = g(x, a, n) 
      y = g(g(y, a, n), a, n)
      d = math.gcd(abs(x-y), n)
    if d == n:
      continue  # keep searching
    return d

def g(x, a, n):
  return (x**2 + a) % n


# Pick a random composite
n = random.randint(3, 2**13)
while pUtils.isPrime(n): 
  n = random.randint(3, 2**13)
print("random composite n =", n)

factor = pollard(n)
print("Factor =", factor)
print("n/factor =", n//factor)
