
# polya.py
# Andrew Davison, ad@coe.psu.ac.th, Nov. 2024
# David Wells - "Prime Numbers": Liouville, Joseph (1809-1882)
  # but not the the divisors of the divisors of a number prop.
# https://en.wikipedia.org/wiki/P%C3%B3lya_conjecture
# https://en.wikipedia.org/wiki/Liouville_function
'''
The Pólya conjecture is that 50% or more of the 
natural numbers less than any given number have an 
odd number of prime factors.

Equivalently, it can be stated in terms of the 
sum of the Liouville lambda function:
    L(n) = \sum _{k=1}^{n}  λ(k) <= 0
for all n > 1. 

The value of λ(n) is:
    λ(n) =  1 if n has an even number of prime factors,
    λ(n) = −1 if n has an odd number of prime factors.

Another definition:
λ(n) = (−1)Ω(n)

Ω(n) is the total number of prime factors of n, counted 
with multiplicity. In other words, if the prime 
factorization of n is:
  n = p1^e1 p2^e2 ... pk^ek
then:
  Ω(n) = e1+e2+...+ek
'''

import math
import pUtils
import matplotlib.pyplot as plt


def liouville(n):
  return (-1) ** len( pUtils.factorizeList(n))

def liouvilleSum(n):
  return sum([liouville(i) for i in range(1, n+1)])

def liouvilleSums(n):
  # a list of all the sums up to n
  tot = 0
  sumTots = []
  for n in range(1, n+1):
    tot += liouville(n)
    sumTots.append(tot)
  return sumTots


limit = 10000

# print("Liouville")
# for i in range(1, 21):
  # print(f"{i:2d}. {liouville(i):>2d}")


xs = range(1, limit+1)
ys = liouvilleSums(limit)

plt.figure(figsize=(10, 6))
plt.plot(xs, ys, label=r'$\sum_{n=1}^{limit} \lambda(n)$')

plt.title(f'Summation of the Liouville Lambda Function up to limit = {limit}')
plt.xlabel('limit')
plt.ylabel(r'$\sum_{n=1}^{limit} \lambda(n)$')
plt.grid(True)
plt.legend()
plt.show()
