
# primeConstant.py
# Andrew Davison, ad@coe.psu.ac.th, Nov. 2024
# https://mathworld.wolfram.com/PrimeConstant.html
'''
Based on the video:
   The Prime Constant - Numberphile
   https://youtu.be/c066hLi78B0
'''

import decimal
from decimal import Decimal as D
import pUtils

def parse_bin(s):
  return D(int(s, 2)) / 2**len(s)


def decimalToBinary(num, prec): 
  '''
    fractional decimal to binary string
    with precision after decimal point
  '''
  binaryStr = "" 
  intPart = int(num) 
  frac = num - intPart 
  
  # Conversion of integer part
  while intPart > 0: 
    rem = intPart % 2
    binaryStr += str(rem); 
    intPart //= 2
  binaryStr = binaryStr[::-1]  # reverse
  if binaryStr == "":    # Append point
    binaryStr += '0.' 
  else:
    binaryStr += '.'
  
  # Conversion of fraction part 
  while prec > 0: 
    # Find next bit in fraction 
    frac *= 2
    fracBit = int(frac) 
    if fracBit == 1: 
      frac -= fracBit 
      binaryStr += '1'
    else : 
      binaryStr += '0'
    prec -= 1
  return binaryStr 


limit = 1000
decimal.getcontext().prec = 100

primes = pUtils.primes(limit)

pDigits = [0]*limit
for p in primes:
  pDigits[p-1] = 1

# print(pDigits)

psStr = ''.join(map(str,pDigits))
print("Primes String:", psStr)
decPrime = parse_bin( psStr)
print("\nBase 10:", decPrime)

binPrecision = 500  # should be less than limit
binStr = decimalToBinary(decPrime, binPrecision)
print("\nBack to binary:", binStr)

binStr = binStr[2:]  # chop "0."
i = 0
while i < binPrecision:
  if psStr[i] != binStr[i]:
    break
  i += 1
print("\nBinary strings differ at pos", i)  # 332
