
# primeCountsLi.py
# Andrew Davison, ad@coe.psu.ac.th, Nov. 2024
# David Wells - "Prime Numbers": prime number theorem and the prime counting function
# https://en.wikipedia.org/wiki/Prime_number
# Also see primeCountsRi.py, liPlot.py

import math
import pUtils
from li import li
import matplotlib.pyplot as plt

def logReciprocal(t):
  return 1 / math.log(t) if t > 1 else 0  # Avoid div by zero


limit = 10
steps = 100000
print("x=10^i    pi(x)  pi(x)-xlx  pi(x)-li(x) (pi(x)/xlx)-1  1-(pi(x)/li(x))")

xs = []
logs = []
lixs = []
for i in range(2, limit):
  x = 10**i
  pix = pUtils.pi(x)
  lix = li(x, steps)
  xlx = x/math.log(x)
  logPercent = (pix/xlx)-1
  lixPercent = 1-(pix/lix)
  print(f"{i:2d}   {pix:8d}   {int(pix-xlx):8d}    {(pix-lix):7d}        {logPercent:6.2%}         {lixPercent:6.2%}")
  xs.append(x)
  logs.append((pix/xlx))
  lixs.append((pix/lix))


plt.figure(figsize=(8, 6))
plt.scatter(xs, logs, label='$\pi(x)/(x/\log x)$', color='blue')
plt.scatter(xs, lixs, label='$\pi(x)$/li(x)', color='red')

plt.axhline(y=1, color='k')
plt.xscale('log')
plt.xlabel('x (log scale)')
plt.ylabel('y')
plt.title('Prime Counting Approxs Ratios')
plt.legend()
plt.show()
