
# primeCountsRi.py
# Andrew Davison, ad@coe.psu.ac.th, Nov. 2024
# David Wells - "Prime Numbers": prime number theorem and the prime counting function
# https://en.wikipedia.org/wiki/Prime_number
# Also see primeCountsLi.py

import math
import pUtils
from li import li
import matplotlib.pyplot as plt


def ri(x):
  # Riemann prime counting function
  if x < 2:
    return 0
  r = li(x)
  for n in range(2, 11, 2):
    r += li(x**(1/n)) * mobius(n)/n
  return int(round(r))


def mobius(n):
  fs = pUtils.primeFactors(n)
  if any(n%(p*p) == 0 for p in fs):
    return 0  # n is divisible by a square prime factor
  return -1 if (len(fs)%2 == 1) else 1



limit = 10
steps = 100000
print("x=10^i    pi(x)  pi(x)-xlx  pi(x)-ri(x) (pi(x)/xlx)-1  1-(pi(x)/ri(x))")

xs = []
logs = []
rixs = []
for i in range(2, limit):
  x = 10**i
  pix = pUtils.pi(x)
  rix = ri(x)
  xlx = x/math.log(x)
  logPercent = (pix/xlx)-1
  rixPercent = 1-(pix/rix)
  print(f"{i:2d}   {pix:8d}   {int(pix-xlx):8d}    {(pix-rix):7d}        {logPercent:6.2%}         {rixPercent:6.2%}")
  xs.append(x)
  logs.append((pix/xlx))
  rixs.append((pix/rix))


plt.figure(figsize=(8, 6))
plt.scatter(xs, logs, label='$\pi(x)/(x/\log x)$', color='blue')
plt.scatter(xs, rixs, label='$\pi(x)$/ri(x)', color='red')

plt.axhline(y=1, color='k')
plt.xscale('log')
plt.xlabel('x (log scale)')
plt.ylabel('y')
plt.title('Prime Counting Approxs Ratios')
plt.legend()
plt.show()

