
# primitiveRoots.py
# Andrew Davison, ad@coe.psu.ac.th, Nov. 2024
# David Wells - "Prime Numbers": primitive roots
# https://en.wikipedia.org/wiki/Artin's_conjecture_on_primitive_roots
# https://en.wikipedia.org/wiki/Primitive_root_modulo_n
# Also see diffieHellman.py

import pUtils
import math

def isPrimitiveRoot(a, p):
  # is 'a' a primitive root modulo 'p'?
  if math.gcd(a, p) != 1:
    return False
  order = set(pow(a, i, p) for i in range(1, p))
  return len(order) == p - 1


limit = 100 
ps = pUtils.primes(limit)
print("Smallest primitive root r for a prime p: (p,r)")
for p in ps:
  for i in range(1, 10):
     if isPrimitiveRoot(i, p):
       print(f"({p},{i}) ", end ='')
       break
print()


a = 2 
prims = []
for p in ps:
  if isPrimitiveRoot(a, p):
    prims.append(p)
print(f"\nPrimes up to {limit} where {a} is a primitive root:")
print(*prims)

