
# pseudoPrimes.py
# Andrew Davison, ad@coe.psu.ac.th, Nov. 2024
# David Wells - "Prime Numbers": pseudoprimes
# https://en.wikipedia.org/wiki/Pseudoprime
# Also see carmichael.py
'''
According to Fermat's Little Theorem, if p is 
prime and a and p are coprime, then
   a^(p-1) ≡ 1 (mod p)
Unfortunately the converse is not true. Sometimes,
   a^(p-1) ≡ 1 (mod n)
although n is composite. For example,
   2^341—1 ≡ 1 (mod 341)
but 341 = 11 * 31
'''

import math
import pUtils


def isPseudoprime(n, a):
  if pUtils.isPrime(n):
    return False  # Only composite numbers are pseudoprime
  return pow(a, n-1, n) == 1


def gen_pps(limit, a):
  return [n for n in range(3, limit+1, 2)
                            if isPseudoprime(n, a)]


def testBase(a, limit):
  # https://en.wikipedia.org/wiki/Fermat_pseudoprime
  print(f"Fermat pseudoprimes to base {a} up to {limit}:")
  print(gen_pps(limit, a))


limit = 1000
testBase(2, limit)
testBase(3, limit)
testBase(5, limit)

