
# pseudoperfect.py
# Andrew Davison, ad@coe.psu.ac.th, Nov. 2024
# # David Wells - "Prime Numbers": pseudoperfect numbers,
#                                  weird numbers
# https://en.wikipedia.org/wiki/Semiperfect_number
# https://oeis.org/A005835  (pseudoperfect)
# https://oeis.org/A006037  (weird)
'''
A semiperfect number, or pseudoperfect number, is a natural 
number n that is equal to the sum of all or some of 
its proper divisors. A semiperfect number that is equal 
to the sum of all its proper divisors is a perfect number. 

A number is weird if it is abundant but not pseudoperfect.
'''


import itertools
import pUtils

def isPerfect(n):
  return sum( pUtils.divisors(n, isProper=True)) == n

def isAbundant(n):
  return sum( pUtils.divisors(n, isProper=True)) > n

def isSemiPerfect(n):
  divs = pUtils.divisors(n, isProper=True)
  for sz in range(len(divs) + 1):
    for combs in itertools.combinations(divs, sz):
      if sum(list(combs)) == n:
        return True
  return False

def isWeird(n):
  return isAbundant(n) and not isSemiPerfect(n)
 


print("Perfects <= 1000:")
for n in range(1, 1001):
  if isPerfect(n):
    print(n, end=' ')
print()

print("\nPseudoperfects <= 100:")
for n in range(1, 101):
  if isSemiPerfect(n):
    print(n, end=' ')
print()

print("\nWeird numbers <= 1000:")
for n in range(1, 1001):
  if isWeird(n):
    print(n, end=' ')
print()
