
# pyprimes.py
# Andrew Davison, ad@coe.psu.ac.th, Nov. 2024
'''
  Python versions of the pyprimesieve functions: 
    * primes(n): 
    * primes(start, n):
    * primes_sum(n).
    * primes_sum(start, n)
    * primes_nth(n)
    * factorize(n)
  This module is used indirectly via functions in
  pUtils.py when the pyprimesieve functions are
  unavailable
'''

import math

def primes(*argv):
  start = 2
  n = 100
  if len(argv) == 0:
    print("Finding primes up to 100")
  elif len(argv) == 1:
    n = argv[0]
  else:
    start = argv[0]
    n = argv[1]

  if n < 2:
    return []
  prime = [True]*n
  prime[0] = False
  prime[1] = False
  for p in range(math.isqrt(n)+1):
    if prime[p]:
      for i in range(p*p, n, p):
        prime[i] = False
  return [p for p in range(start, n) if prime[p]]


def primes_sum(*argv):
  start = 2
  n = 100
  if len(argv) == 0:
    print("Finding sum of primes up to 100")
  elif len(argv) == 1:
    n = argv[0]
  else:
    start = argv[0]
    n = argv[1]
  return sum(primes(start, n))


def primes_nth(n):
  prims = [2]
  num = 3
  while len(prims) < n:
    for p in prims:
      if num % p == 0:
        break
    else:
      prims.append(num)
    num += 2
  return prims[-1]



def factorize(n):
  factors = {}
  d = 2
  while n > 1:
    while n%d == 0:
      if d in factors:
        factors[d] += 1
      else:
        factors[d] = 1
      n /= d
    d = d + 1
    if d * d > n:
      if n > 1:
        n = int(n)
        if d in factors:
          factors[n] += 1
        else:
          factors[n] = 1
        break
  return [(k, v) for k, v in factors.items()]



if __name__ == "__main__":
  print(primes(11))
  print(primes(97))
  print(primes(71,97))
  print(primes_sum(11))
  print(primes_sum(5, 11))
  print(primes_nth(1))
  print(primes_nth(4))
  print(factorize(120))
