
# pythPrime.py
# Andrew Davison, ad@coe.psu.ac.th, Nov. 2024
# David Wells - "Prime Numbers": Pythagorean triangles, prime
# https://t5k.org/glossary/page.php?sort=PrmPythagTriples

import math
import pUtils

def genTriplets(limit):
  # generate Pythagorean triplets using the 
  # 2mn, m^2 - n^2, and m^2 + n^2 formulas
  triplets = []
  for m in range(2, limit):
    for n in range(1, m):
      a = m**2 - n**2
      b = 2*m*n
      c = m**2 + n**2
      if c > limit:
        break
      triplets.append((a, b, c))
  return triplets


def getPrimes(triplets):
  # select if one leg and the hypotenuse are prime
  ps = []
  for triplet in triplets:
    a, b, c = triplet
    if (pUtils.isPrime(a) or pUtils.isPrime(b)) and \
       pUtils.isPrime(c):
      ps.append(triplet)
  return ps


limit = 10000
triplets = genTriplets(limit)
ps = getPrimes(triplets)
print("Pythagorean triplets where one leg and hypotenuse are prime:")
for triplet in ps:
  print(' ', triplet)


