
# quadRecip.py
# Andrew Davison, ad@coe.psu.ac.th, Nov. 2024
# David Wells - "Prime Numbers": quadratic reciprocity, law of
# https://en.wikipedia.org/wiki/Quadratic_reciprocity
# https://en.wikipedia.org/wiki/Legendre_symbol
# Also see jacobiSymbol.py
# inputs:  7 11  or  6 7  or  152 43

import pUtils

def legendreSymbol(a, p):  # (a|p)
  if (p%2 == 0) or not pUtils.isPrime(p):
    print(f"p in ({a}|{p}) must be an odd prime")
    return 0
  if a % p == 0:
    return 0
  else:   
    res = pow(a, (p-1)//2, p)  # 0, 1, or p-1
    return res-p if res > 1 else res


'''
a, p = map(int, input("a p for (a|p)? ").split())
apQR = legendreSymbol(a,p)  # (a|p)
print(f"({a}|{p}): {apQR}")
'''

aVals = list(range(11))
ps = pUtils.primes(3, 20)

print("Value of the Legendre Symbol (a|p):")
header = "p\\a " + "  ".join(f"{col:>3}"  for col in aVals)
print(header)

for p in ps:
  row = [legendreSymbol(a, p) for a in aVals]
  print(f"{p:>2}  " + "  ".join(f"{num:>3}" for num in row))



