
# randPrimes.py
# Andrew Davison, ad@coe.psu.ac.th, Nov. 2024
# David Wells - "Prime Numbers": randomness, of primes
# https://en.wikipedia.org/wiki/Prime_omega_function
# https://www.johndcook.com/blog/2023/06/24/little-omega/
# https://www.johndcook.com/blog/2016/06/22/prime-factors-phone-numbers-and-the-normal-distribution/

import math
from collections import Counter
import pUtils
import matplotlib.pyplot as plt


def omega(n):
  # counts each distinct prime factor
  return len(pUtils.primeFactors(n))


def avgOmega(limit):
  tot = sum( [omega(i) for i in range(1, limit+1)] )
  return tot/limit


print("n = 10^i avg omega(n) log(log(n))")
for i in range(3,7):
  n = 10**i
  print(f"    {i}      {avgOmega(n):.4f}       {math.log(math.log(n)):.4f}")

# ---------------------------------------

limit = 100000
oms = [omega(n) for n in range(2, limit)]

counts = Counter(oms)
numbersPFs = list(counts.keys())
countOmegas = list(counts.values())
plt.bar(numbersPFs, countOmegas)

plt.xlabel('No. of distinct prime factors')
plt.ylabel('Counts')
plt.title(f'Counts of distinct prime factors for n < {limit}')
plt.show()