
# redmondSun.py
# Andrew Davison, ad@coe.psu.ac.th, Nov. 2024
# https://ia803203.us.archive.org/7/items/conjectures-about-prime-numbers/Redmond%E2%80%93Sun_conjecture.pdf
# https://planetmath.org/redmondsunconjecture
'''
The Redmond–Sun conjecture states that every interval 
[x^m, y^n] with x, y, m, n in {2, 3, 4, ...} and 
x^m /= y^n contains primes with only finitely many 
exceptions. Namely, those exceptional intervals 
[x^m, y^n] are:

1.There are no primes between 2^3 and 3^2.
2.There are no primes between 5^2 and 3^3.
3.There are no primes between 2^5 and 6^2.
4.There are no primes between 11^2 and 5^3.
5.There are no primes between 3^7 and 13^3.
6.There are no primes between 5^5 and 56^2.
7.There are no primes between 181^2 and 2^15. (32^3 and 8^5)
8.There are no primes between 43^3 and 282^2.
9.There are no primes between 46^3 and 312^2.
10.There are no primes between 22434^2 and 55^5.  

See OEIS A116086 i(https://oeis.org/A116086) for a 
listing of the perfect powers beginning primeless 
ranges before the next perfect power.  
'''

import math
import pUtils

MAX = 400
powers = {2, 3, 4, 5}

def primesRange(start, end):
  for num in range(start, end+1):
    if pUtils.isPrime(num):
      return True
  return False


print(f"x, y <= {MAX}; powers from {powers}:")
for x in range(2, MAX+1):    # inefficient
  for y in range(2, MAX+1):
    for m in powers:
      for n in powers:
        pow1 = int(math.pow(x, m))
        pow2 = int(math.pow(y, n))
        if pow1 < pow2:
          if not primesRange(pow1, pow2):
          # ps = pUtils.primes(pow1, pow2)
          # if ps == []:
            print(f"No primes between {x}^{m} and {y}^{n}")
