
# rieFarey.py
# Andrew Davison, ad@coe.psu.ac.th, Nov. 2024
# David Wells - "Prime Numbers": the Riemann hypothesis; the Farey sequence
# https://en.wikipedia.org/wiki/Farey_sequence#Riemann_hypothesis
# https://www.maths.ed.ac.uk/~v1ranick/fareyproject.pdf (ch. 5)
# input:  5  or 50  or 100

from fractions import Fraction as F
import math

def fareySeq(n):
  seq = [F(0, 1), F(1, 1)]  
  for denom in range(2, n+1):
    for numer in range(1, denom):
      f = F(numer, denom)
      if f not in seq:
        seq.append(f)
  return sorted(seq)


n = int(input("n? "))  # Farey seq order
fs = fareySeq(n)
# print(*fs)

sp = len(fs)-1
esFracs = [ F(1,sp)*i for i in range(sp+1)]
# equally spaced fractions on the unit interval
# print(*esFracs)

diffs = [abs(f-e) for f, e in zip(fs, esFracs)]
# print(*diffs)
S = float(sum(diffs))
print("Sum of diffs:", S)
print("\nS/(n^0.5):", S/(math.sqrt(n)))
   # seems to approach 0.25 ??
