
# rsa.py
# Andrew Davison, ad@coe.psu.ac.th, Nov. 2024
# David Wells - "Prime Numbers": RSA algorithm
# https://www.askpython.com/python/examples/rsa-algorithm-in-python
# https://www.geeksforgeeks.org/rsa-algorithm-cryptography/
# Also see diffieHelman.py
'''
Alice generates a private key and public key.  
The code shows how Bob can encrypt a message to Alice 
using her public key, and how Alice can decrypt it with
her private key. 
'''

import random, math
import pUtils

def genKeys(p, q):
  n = p*q
  totient = (p-1)*(q-1)  # phi()
  while True:
    e = random.randint(3, n-2)
    if math.gcd(e, totient) == 1:
      break  # e is valid
  d = pUtils.modInverse(e, totient)  
  return d, n, e  #  private-key d, n, public-key e


# Choose two random primes p, q (fairly close together)
k = random.randint(2**20, 2**25)
p = pUtils.prime(k)
q = pUtils.prime(k+random.randint(-30, 30))

d, n, e = genKeys(p, q)
print("Public key (e, n) =", (e, n))
print("Private key (d, n) =", (d, n))
print()

# --------------------------------

# Pick random primes for Alice
p = pUtils.prime(random.randint(64, 100)) 
q = pUtils.prime(p+random.randint(-20, 20))

d, n, e = genKeys(p, q)
print("random prime p =", p)
print("random prime q =", q)
print("Public key (e, n) =", (e, n))
print("Private key (d, n) =", (d, n))

# Bob can access Alice's public key, (e, n)
# Bob sends alice a message using it
message = random.randint(0, 2**10)
encoded = pow(message, e, n)

# Alice decodes it with her private key (d, n)
decoded = pow(encoded, d, n)

print("Original message =", message)
print("Encoded message =", encoded)
print("Decoded message =", decoded)
