
# scherk.py
# Andrew Davison, ad@coe.psu.ac.th, Nov. 2024
# David Wells - "Prime Numbers": Scherk's conjecture
# https://hal.science/hal-01936230/document
# exponential running time
# input: 29   or  71   or 31

import itertools
import pUtils


def addSubCombos(nums):
  # Generate all combos of + and - with nums
  signs = list(itertools.product([1, -1],  
              repeat=len(nums)))  # (1 for +, -1 for -)
               # the cartesian product of signs
  # print(signs)
  exprs = []
  for signSet in signs:
    expr = []
    total = 0
    for num, sign in zip(nums, signSet):
      if sign == 1:
        expr.append(f"+{num}")
      else:
        expr.append(f"-{num}")
      total += sign * num
    expStr = (total, "".join(expr))  # store total and expr
    exprs.append(expStr)
  return exprs


print("Even index primes:", pUtils.primes(80)[1::2])

p = int(input("Select one? "))
prims = pUtils.primes(p)  # all primes smaller than p
exprs = addSubCombos([1]+prims)

# test all the possible exprs until we get one == p
sumFound = False
for total, expr in exprs:
  if total == p:
    print(f"{expr} = {total}")
    sumFound = True
    break
if not sumFound:
  print("No sum of primes for", p)