
# shanks.py
# Andrew Davison, ad@coe.psu.ac.th, Nov. 2024
# David Wells - "Prime Numbers": Shank's conjecture
# https://mathworld.wolfram.com/PrimeGaps.html
# https://mathworld.wolfram.com/ShanksConjecture.html

import math
import pUtils
import matplotlib.pyplot as plt


maxGap = 140  # even
limit = 1000000

print(f"Examining primes < {limit}:")
prims = pUtils.primes(limit)
print(*prims[:15], "...")

# using Wolfram definitions
print(f"\nSearching for even gaps <= {maxGap}:")
smallestGPs = []
for gap in range(2, maxGap+1, 2):
  for i in range(len(prims)-1):
    if prims[i+1] - prims[i] == gap:
      smallestGPs.append((gap, prims[i]))  # (g, p(g))
      break
print("(g, p(g)) pairs:", *smallestGPs)


# print("\n g       p(g)    log(p(g))**2")
gs = []
approxs = []
for g, pg in smallestGPs:
   # approx = pg/(math.sqrt(g)*math.exp(math.sqrt(g)))
   approx = (math.log(pg)**2)
   # print(f"{g:3d}   {pg:6d}    {approx:>8.1f}")
   gs.append(g)
   approxs.append(approx)

plt.plot(gs, approxs)
plt.xlabel('Gap g')
plt.ylabel('log(p(g))^2')
plt.title('Smallest prime gaps vs. sqr log primes')
plt.show()
  # see more detailed graph at 
  #  https://mathworld.wolfram.com/PrimeGaps.html
