
# smith.py
# Andrew Davison, ad@coe.psu.ac.th, Nov. 2024
# David Wells - "Prime Numbers": Smith numbers
# https://en.wikipedia.org/wiki/Smith_number
# https://oeis.org/A006753
'''
A Smith number is a composite number for which the sum 
of its digits is equal to the sum of the digits in its 
prime factorization (counting multiplicities). 
'''

import pUtils

def isSmith(n):
  if n < 2:
    return False

  facs = pUtils.factorizeList(n)
  if len(facs) == 1:  
    # Prime numbers are not Smith numbers
    return False
  
  sd_number = pUtils.sumDigits(n)
  sd_factors = sum( pUtils.sumDigits(f) for f in facs)
  return sd_number == sd_factors



limit = 1000
smiths = []
for num in range(2, limit+1):
  if isSmith(num):
    smiths.append(num)
print("Smith numbers up to", limit, ":", *smiths)
