
# spiral.py
# Andrew Davison, ad@coe.psu.ac.th, Nov. 2024
'''
The patterns relate to the rational approximation of 2*pi

See the video:
  "Why do prime numbers make these spirals? | 
  Dirichlet’s theorem and pi approximations", 3Blue1Brown
  https://www.youtube.com/watch?v=EK32jo7i5LQ

https://numberspiral.com/index.html

Also see spiralSacks, spiralUlam.py
'''

import math
import pUtils
import matplotlib.pyplot as plt


def plotNums(nums, figsize=8, s=None):
  xs = []
  ys = []
  for n in nums:
    xs.append(n * math.cos(n))
    ys.append(n * math.sin(n))
  plt.figure(figsize=(8, 8))
  plt.axis("off")
  plt.scatter(xs, ys, s=s, c='black')
  plt.show()


# with more numbers, reduce the point size

# primes = pUtils.primes(500)
# plotNums(primes)

primes = pUtils.primes(20000)
plotNums(primes, s=1)

# primes = pUtils.primes(400000)
# plotNums(primes, s=0.15)

# -------------------
# spirals appear for any sequence:

# nums = range(1000)
# plotNums(nums)

# nums = range(10000)
# plotNums(nums, s=1)
