
# squarefree.py
# Andrew Davison, ad@coe.psu.ac.th, Nov. 2024
# David Wells - "Prime Numbers": squarefree numbers
# https://en.wikipedia.org/wiki/Square-free_integer
'''
A squarefree number is a number that is not divisible 
by any perfect square other than 1. This means thst
none of its prime factors appear more than once in 
its prime factorization.
'''

import math
import pUtils

def isSquareFree(n): 
  fvals = pUtils.factorizeDict(n).values()
  return all(v == 1 for v in fvals)


limit = 100000
sfNums = []
for num in range(1, limit+1):
  if isSquareFree(num):
    sfNums.append(num)

print("First few squarefree numbers:")
print(" ", *sfNums[:20], "...")
print("There are", len(sfNums), "squarefree numbers up to", limit)
print(f"\nNote: int 6*{limit}/pi^2 =", 
         int(round((6*limit)/(math.pi*math.pi))))
