
# timeout.py
# Andrew Davison, ad@coe.psu.ac.th, Nov. 2024
'''
  A timeout() function for interupteding its supplied
  function after a specified amount of time, with the
  option of executing a backup function instead.

  Due to Windows's fork system call, Python's
  multiprocessing module on Windows imports the script 
  once for each process. This means that any IO outside 
  of a main block will be duplicated.

  As a result, I've decided not to include support for
  timeout() in pUtils.py, although there is some 
  commented-out code in there).
'''


import multiprocessing
import time
import pUtils


def fnWrapper(queue, fn, *args):
  result = fn(*args) 
  queue.put(result)


def timeout(func, backupFunc=None, args=(), timeout=5):
  queue = multiprocessing.SimpleQueue()
  
  process = multiprocessing.Process(target=fnWrapper, 
                  args=(queue, func, *args,))
  process.start()
  process.join(timeout)
  if process.is_alive():
    # print(f"Timeout after {timeout} secs.")
    process.terminate()
    process.join()
    if backupFunc:
      res = backupFunc(*args)
      queue.put(res)
    else:
      queue.put(None) 
  if not queue.empty():
    return queue.get()
  else:
    return None 


def test1(x):
  time.sleep(3)  # Simulate work
  return f"test1 == {x}"

def backup1(x):
  return f"backup1 == {x}"


if __name__ == '__main__':
  # result = timeout(test1, backup1, args=(5,), timeout=10)
  # print("Result:", result)
  
  start =time.time()
  print(pUtils.isPrime(11111111111111111131))
  print(f"Elapsed time: {(time.time()-start):.2f} secs")  
    # too large for C; and takes 14 secs in pyprimes

  result = timeout(pUtils.isPrimeSLOW, pUtils.isPrimeMR, 
                       args=(11111111111111111131,))
  print("Result:", result)

