
# truncatable.py
# Andrew Davison, ad@coe.psu.ac.th, Nov. 2024
# David Wells - "Prime Numbers": left-truncatable prime, right-truncatable prime, deletable and truncatable primes
# https://en.wikipedia.org/wiki/Truncatable_prime
# https://mathworld.wolfram.com/TruncatablePrime.html


import pUtils

def isTruncatablePrime(n, mode='left'):
  nStr = str(n)
  if '0' in nStr:
    return False
  while n != None:
    if not pUtils.isPrime(n):
      return False
    n = truncate(n, mode)
  return True


def truncate(n, mode='left'):
  nStr = str(n)
  if len(nStr) == 1:
    return None
  else:
    if mode == 'left':
      return int(nStr[1:])
    else:
      return int(nStr[:-1])


# print( truncate(12345))
# print( truncate(12345, mode='right'))

limit = 10000
lefts = []
rights = []
for i in range(2, limit+1):
   if isTruncatablePrime(i, 'left'):
     lefts.append(i)
   if isTruncatablePrime(i, 'right'):
     rights.append(i)

print(f"\n{len(lefts)} left-truncatable primes <= {limit}:")
print(*lefts)

print(f"\n{len(rights)} right-truncatable primes <= {limit}:")
print(*rights)

inBoth = [v for v in lefts if v in rights]
print(f"\n{len(inBoth)} left/right-truncatable primes <= {limit}:")
print(*inBoth)

