
# unitary.py
# Andrew Davison, ad@coe.psu.ac.th, Nov. 2024
# David Wells - "Prime Numbers": unitary divisors
# https://en.wikipedia.org/wiki/Unitary_divisor
# https://en.wikipedia.org/wiki/Unitary_perfect_number
# input: 60  or  17
'''
  A divisor d of n is a unitary divisor if gcd⁡(d, n/d)=1,
'''

import math
import pUtils

def unitaryDivs(n):
  divs = pUtils.divisors(n)
  uDivs = []
  for d in divs:
    if math.gcd(d, n//d) == 1:
      uDivs.append(d)
  return uDivs


def isUnitaryPerfect(n):
  uDivs = unitaryDivs(n)
  uDivs.remove(n)
  return n == sum(uDivs)


n = int(input("n? "))
print("Unitary divisors:")
print(unitaryDivs(n))

print("\nUnitary perfects up to 1000:")
for i in range(1, 1001):
  if isUnitaryPerfect(i):
    print(" ", i, end='')
print()
