
# untouchable.py
# Andrew Davison, ad@coe.psu.ac.th, Nov. 2024
# David Wells - "Prime Numbers": untouchable numbers
# https://en.wikipedia.org/wiki/Untouchable_number
# https://oeis.org/A005114
# input: 60  or  17
'''
An untouchable number is any number that cannot be the sum of 
aliquot parts (proper divisors) of n. In other words, it is never 
a value of sigma(n) - n. 

The sequence starts, 2, 5, 52, 88, 96, 120, 124, 146, 162, 178, ... 
(Sloane A005114)
'''

import math, itertools
import pUtils

limit = 300
aliSums2 = [ (pUtils.sigma(i) - i) for i in range(1, limit**2)]
aliSums = list(itertools.takewhile(lambda x: x <= limit, 
                              sorted(set(aliSums2))))
# print(f"Sum of aliquot parts <= {limit}:")
# print(aliSums)

print(f"\nUntouchables <= {limit}:")
untouchs = c = [n for n in range(1,limit+1) 
                          if n not in aliSums]
print(*untouchs)


