
# wolstenholme.py
# Andrew Davison, ad@coe.psu.ac.th, Nov. 2024
# David Wells - "Prime Numbers": Wolstenholme’s numbers, and theorems
# https://en.wikipedia.org/wiki/Wolstenholme_number
# https://mathworld.wolfram.com/WolstenholmeNumber.html

import pUtils
from fractions import Fraction as F

def wolstenholme(n):
  hs = sum([F(1, k**2) for k in range(1, n+1)])
  return hs.numerator

limit = 50
print(f"First {limit} Wolstenholme numbers:")
wNums = [wolstenholme(n) for n in range(1, limit+1)]
print(*wNums[:10], "...")

# Wolstenholme’s theorem
ps = pUtils.primes(5, limit)
print("\nChecking Wolstenholme's theorem:")
for i in range(len(ps)):
  print(f"Does {ps[i]} divide WN_{ps[i]-1}?",
                           wNums[ps[i]-2]%ps[i] == 0)