
# wolstenholmePs.py
# Andrew Davison, ad@coe.psu.ac.th, Nov. 2024
# David Wells - "Prime Numbers": Wolstenholme’s numbers, and theorems
# https://en.wikipedia.org/wiki/Wolstenholme_prime
# https://oeis.org/A088164
'''
A Wolstenholme prime is a prime > 7 that satisfies
  ( 2p−1 p−1 ) ≡ 1 (mod p^4)
where the (...) expression on the left is a 
binomial coefficient.

Are there any Wolstenholme primes other than 16843 
and 2124679?
'''

import math
import pUtils

def isWolstenholme(p):
  return (math.comb(2*p-1,p-1) % (p**4)) == 1


limit = 20000
prims = pUtils.primes(limit)
print(f"Wolstenholme primes 2-{limit}:")
for p in prims:
  if isWolstenholme(p):
    print(p, end=' ')
print()   # 16843  -- takes a while

